@@ -1,7 +1,5 @@
 package Treex::Block::Read::BaseReader;
-{
-  $Treex::Block::Read::BaseReader::VERSION = '0.08664';
-}
+$Treex::Block::Read::BaseReader::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Common;
 use File::Slurp;
@@ -13,18 +11,14 @@ sub next_document {
     return log_fatal "method next_document must be overridden in " . ref($self);
 }
 
+# Default language is und (undetermined/unknown) and selector empty.
 has selector => ( isa => 'Treex::Type::Selector', is => 'ro', default => q{} );
-
-# Note that "language" is intentionally not required here,
-# because some derived classes (e.g. Read::Treex) do not need it.
-# However, if a derived class reads a format where the language is not specified,
-# it must add the parameter language using:
-#has language => ( isa => 'Treex::Type::LangCode', is => 'ro', required=>1 );
+has language => ( isa => 'Treex::Type::LangCode', is => 'ro', default => 'und' );
 
 
 has from => (
     isa           => 'Treex::Core::Files',
-    is            => 'ro',
+    is            => 'rw',
     coerce        => 1,
     required      => 1,
     handles       => [qw(current_filename file_number _set_file_number)],
@@ -47,26 +41,59 @@ has is_one_doc_per_file => (
 
 has _file_numbers => ( is => 'rw', default => sub { {} } );
 
-sub is_next_document_for_this_job {
-    my ($self) = @_;
-    return 1 if !$self->jobindex;
-    return $self->doc_number % $self->jobs == ( $self->jobindex - 1 );
+has _file_number_width => (
+    is => 'rw',
+    isa => 'Int',
+    default => 3,
+    documentation => 'The number of digits for numbered filenames.  '
+        . 'The default (3) will create filenames with three digits as "001.treex.gz".'
+);
+
+has skip_finished => (
+    isa           => 'Str',
+    is            => 'ro',
+    documentation => 'Skip input files for which a matching non-empty output file exists '
+        . '(presumably created by a previous unfinished Treex run). '
+        . 'This parameter specifies a regex substitution how to derive the output filename from the input filename. '
+        . 'It is parallel to the parameter substitute={indir}{outdir} in writers. '
+        . 'However, you need to take care of filename extensions too, '
+        . 'e.g. if converting conll to treex, you should use skip_finished={indir/(.+).conll$}{outdir/$1.treex.gz}',
+);
+
+
+sub BUILD {
+    my ( $self, $args ) = @_;
+    if (my $regex = $self->skip_finished){
+        my $filenames_ref = $self->from->filenames;
+        my @filtered_filenames;
+        my $eval_string = '$filename =~ s' . $regex . '; 1;';
+
+        for my $input_filename (@$filenames_ref){
+            my $filename = $input_filename;
+
+            # see r14228 for an alternative implementation (without stringy eval) which cannot handle $1 in rexex
+            eval $eval_string or log_fatal "Failed to eval $eval_string"; ## no critic qw(BuiltinFunctions::ProhibitStringyEval)
+
+            if (! -s $filename){
+                push @filtered_filenames, $input_filename;
+                #say "not finished: $input_filename -> $filename";
+            } #else {say "finished: $input_filename -> $filename";}
+        }
+        $self->from->_set_filenames(\@filtered_filenames);
+        my $input_number = @$filenames_ref;
+        my $filtered_number = @filtered_filenames;
+        my $finished_number = $input_number - $filtered_number;
+        log_info "$finished_number files out of $input_number were finished, reading only the remaining $filtered_number.";
+    }
+    return;
 }
 
 sub next_filename {
-    
     my ($self) = @_;
 
-    # In parallel processing and one_doc_per_file setting,
-    # we can skip files that are not supposed to be loaded by this job/reader,
-    # in order to make loading faster.
-    while ( $self->is_one_doc_per_file && !$self->is_next_document_for_this_job ) {
-        $self->_set_file_number( $self->file_number + 1 );
-        $self->_set_doc_number( $self->doc_number + 1 );
-    }
-    # return undef, but do not move further if we are at the end of document list (we might need the current file name) 
+    # return undef, but do not move further if we are at the end of document list (we might need the current file name)
     return if ( $self->file_number >= $self->from->number_of_files );
-    
+
     $self->_set_file_number( $self->file_number + 1 );
     return $self->current_filename();
 }
@@ -74,18 +101,25 @@ sub next_filename {
 use File::Spec;
 
 sub new_document {
-    
     my ( $self, $load_from ) = @_;
     my $path = $self->current_filename();
     log_fatal "next_filename() must be called before new_document()" if !defined $path;
     my ( $volume, $dirs, $file ) = File::Spec->splitpath($path);
-    my ( $stem, $extension ) = $file =~ /([^.]+)(\..+)?/;
-    $stem =~ s/^-$/noname/;
-    my %args = ( file_stem => $stem, loaded_from => $path );
+
+    # Delete file extension, e.g.
+    # file.01.conll -> file.01
+    # cs42.treex.gz -> cs42
+    $file =~ s/\.[^.]+(\.gz)?$//;
+
+    # Substitute standard input for noname.
+    $file =~ s/^-$/noname/;
+
+    my %args = ( file_stem => $file, loaded_from => $path );
     if ( defined $dirs ) {
         $args{path} = $volume . $dirs;
     }
 
+    # Override the naming heuristics above, if file_stem was specified.
     if ( $self->file_stem ) {
         $args{file_stem} = $self->file_stem;
     }
@@ -94,9 +128,10 @@ sub new_document {
         $args{file_number} = q{};
     }
     else {
-        my $num = $self->_file_numbers->{$stem};
-        $self->_file_numbers->{$stem} = ++$num;
-        $args{file_number} = sprintf "%03d", $num;
+        my $num = $self->_file_numbers->{$file};
+        $self->_file_numbers->{$file} = ++$num;
+        my $fmt = "%0".$self->_file_number_width."d";
+        $args{file_number} = sprintf $fmt, $num;
     }
 
     if ( defined $load_from ) {
@@ -135,18 +170,22 @@ after 'restart' => sub {
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
 Treex::Block::Read::BaseReader - abstract ancestor for document readers
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
-This class serves as an common ancestor for document readers,
-that have a parameter C<from> with a space or comma separated list of filenames
+This class serves as a common ancestor for document readers
+that have the parameter C<from> with a space or comma separated list of filenames
 to be loaded.
 It is designed to implement the L<Treex::Core::DocumentReader> interface.
 
@@ -163,8 +202,16 @@ space or comma separated list of filenames, or C<-> for STDIN
 
 An '@' directly in front of a file name causes this file to be interpreted as a file
 list, with one file name per line, e.g. '@filelist.txt' causes the reader to open
-'filelist.txt' and read a list of files from it. File lists may be arbitrarily 
-mixed with regular files in the parameter. 
+'filelist.txt' and read a list of files from it. File lists may be arbitrarily
+mixed with regular files in the parameter.
+
+Similarly, you can use I<!> for wildcard expansion, e.g.
+C<treex -Len Read::Treex from='!dir??/file*.txt'>.
+The single quotes are needed for two reasons.
+First, to prevent bash from interpreting the wildcard characters.
+Second, to prevent bash from interpreting the exclamation mark as history expansion.
+
+The I<@filelist> and I<!wildcard> conventions are used in several tools, e.g. 7z or javac.
 
 (If you use this method via API you can specify a string array reference or a
 L<Treex::Core::Files> object.)
@@ -200,7 +247,7 @@ which are guessed based on C<current_filename>.
 
 =item current_filename
 
-returns the last filename returned by C<next_filename> 
+returns the last filename returned by C<next_filename>
 
 =item is_next_document_for_this_job
 
