@@ -1,7 +1,5 @@
 package Treex::Block::Read::BaseTextReader;
-{
-  $Treex::Block::Read::BaseTextReader::VERSION = '0.08664';
-}
+$Treex::Block::Read::BaseTextReader::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Common;
 extends 'Treex::Block::Read::BaseReader';
@@ -9,15 +7,15 @@ extends 'Treex::Block::Read::BaseReader';
 use PerlIO::via::gzip;
 
 # By default read from STDIN
-has '+from' => ( default => '-' );
+has '+from' => (
+    default => '-',
+    handles => [qw(current_filename current_filehandle file_number _set_file_number next_filehandle)],
+);
 
-has language      => ( isa => 'Treex::Type::LangCode', is => 'ro', required => 1 );
 has lines_per_doc => ( isa => 'Int',                   is => 'ro', default  => 0 );
 has merge_files   => ( isa => 'Bool',                  is => 'ro', default  => 0 );
 has encoding      => ( isa => 'Str',                   is => 'ro', default  => 'utf8' );
 
-has _current_fh => ( is => 'rw' );
-
 sub BUILD {
     my ($self) = @_;
     if ( $self->lines_per_doc ) {
@@ -26,50 +24,22 @@ sub BUILD {
     return;
 }
 
-sub next_filehandle {
-
+sub next_document_text {   
     my ($self) = @_;
-    my $filename = $self->next_filename();
-    return if !defined $filename;
-    if ( $filename eq '-' ) {
-        binmode STDIN, $self->encoding;
-        return \*STDIN;
-    }
-    my $mode = $filename =~ /[.]gz$/ ? '<:via(gzip):' : '<:';
-    $mode .= $self->encoding;
-    open my $FH, $mode, $filename or log_fatal "Can't open $filename: $!";
-    return $FH;
-}
-
-sub next_document_text {
-    
-    my ($self) = @_;
-    my $FH = $self->_current_fh;
-    if ( !$FH ) {
-        $FH = $self->next_filehandle() or return;
-        $self->_set_current_fh($FH);
-    }
-
     if ( $self->is_one_doc_per_file ) {
-        $self->_set_current_fh(undef);
-        local $/ = undef;
-        return <$FH>;
+        return $self->from->next_file_text();
     }
 
     my $text = '';
     LINE:
     for my $line ( 1 .. $self->lines_per_doc ) {
-        while ( eof($FH) ) {
-            $FH = $self->next_filehandle();
-            if ( !$FH ) {
-                return if $text eq '';
-                return $text;
-            }
-            $self->_set_current_fh($FH);
-            last LINE if !$self->merge_files;
+        $line = $self->from->next_line();
+        if (!defined $line){
+            return if $text eq '' && !$self->from->has_next_file();
+            last LINE;
         }
         
-        $text .= <$FH>;
+        $text .= $line;
     }
     return $text;
 }
@@ -78,6 +48,10 @@ sub next_document_text {
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =for Pod::Coverage BUILD
 
 =head1 NAME
@@ -86,7 +60,7 @@ Treex::Block::Read::BaseTextReader - abstract ancestor for document readers
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
@@ -148,6 +122,6 @@ Martin Popel
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
+Copyright © 2011-2012 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
 This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
