@@ -1,11 +1,11 @@
 package Treex::Block::Read::Sentences;
-{
-  $Treex::Block::Read::Sentences::VERSION = '0.08664';
-}
+$Treex::Block::Read::Sentences::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Common;
 extends 'Treex::Block::Read::BaseTextReader';
 
+has 'skip_empty' => (is => 'ro', isa => 'Bool', default => 0);
+
 sub next_document {
     my ($self) = @_;
     my $text = $self->next_document_text();
@@ -13,6 +13,7 @@ sub next_document {
 
     my $document = $self->new_document();
     foreach my $sentence ( split /\n/, $text ) {
+        next if ($sentence eq '' and $self->skip_empty);
         my $bundle = $document->create_bundle();
         my $zone = $bundle->create_zone( $self->language, $self->selector );
         $zone->set_sentence($sentence);
@@ -25,13 +26,21 @@ sub next_document {
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
 Treex::Block::Read::Sentences
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
+
+=head1 SYNOPSIS
+
+ Read::Sentences from='!dir*/file*.txt' skip_empty=1
 
 =head1 DESCRIPTION
 
@@ -46,6 +55,11 @@ L<document|Treex::Core::Document>.
 =item from
 
 space or comma separated list of filenames
+See L<Treex::Core::Files> for full syntax.
+
+=item skip_empty
+
+If set to 1, ignore empty lines (don't create empty sentences). 
 
 =back
 
@@ -65,7 +79,7 @@ L<Treex::Block::Read::BaseTextReader>
 L<Treex::Core::Document>
 L<Treex::Core::Bundle>
 L<Treex::Block::Read::AlignedSentences>
-
+L<Treex::Block::Read::SentencesTSV>
 
 =head1 AUTHOR
 
