@@ -1,7 +1,5 @@
 package Treex::Block::Write::BaseTextWriter;
-{
-  $Treex::Block::Write::BaseTextWriter::VERSION = '0.08664';
-}
+$Treex::Block::Write::BaseTextWriter::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Common;
 use autodie;
@@ -44,7 +42,7 @@ sub _build_to {
 
     my ($self) = @_;
 
-    if ( !defined( $self->path ) && !defined( $self->file_stem ) ) {
+    if ( !defined( $self->path ) && !defined( $self->file_stem ) && !defined( $self->substitute ) ) {
         return '-';
     }
     return;
@@ -59,25 +57,23 @@ override '_get_next_filename' => sub {
     return super();
 };
 
-# Default process_document method for all Writer blocks.
-override 'process_document' => sub {
-
-    my ( $self, $document ) = @_;
-
-    # set _file_handle properly (this MUST be called if process_document is overridden)
-    $self->_prepare_file_handle($document);
-
+sub _do_before_process {
+    my ($self, $document) = @_;
     # allow header printing (in overrides)
     $self->print_header($document);
 
-    # call the original process_document with _file_handle set
-    $self->Treex::Core::Block::process_document($document);
+    return;
+}
+
+sub _do_after_process {
+    my ($self, $document) = @_;
 
     # allow footer printing (in overrides)
     $self->print_footer($document);
 
     return;
-};
+}
+
 
 sub print_header {
     my ( $self, $document ) = @_;
@@ -101,7 +97,7 @@ Treex::Block::Write::BaseTextWriter
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
