@@ -1,25 +1,17 @@
 package Treex::Block::Write::Text;
-{
-  $Treex::Block::Write::Text::VERSION = '0.08664';
-}
+$Treex::Block::Write::Text::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Common;
 extends 'Treex::Block::Write::BaseTextWriter';
 
-has '+language' => ( required => 1 );
-
 has '+extension' => ( default => '.txt' );
 
-override 'process_document' => sub {
-  
+override '_do_process_document' => sub { 
     my ( $self, $doc ) = @_;
 
-    $self->_prepare_file_handle($doc);    # open the output file handle
-
-    my $doczone = $doc->get_zone( $self->language, $self->selector );
-  
-    print { $self->_file_handle } $doczone->text;
-  
+    foreach my $doczone ($self->get_selected_zones($doc->get_all_zones())){
+        print { $self->_file_handle } $doczone->text;
+    }  
     return;
 };
 
@@ -27,13 +19,17 @@ override 'process_document' => sub {
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
 Treex::Block::Write::Text
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
