@@ -1,9 +1,9 @@
 package Treex::Block::Write::Treex;
-{
-  $Treex::Block::Write::Treex::VERSION = '0.08664';
-}
+$Treex::Block::Write::Treex::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Common;
+use File::Path;
+use File::Basename;
 
 extends 'Treex::Block::Write::BaseWriter';
 
@@ -13,7 +13,7 @@ has '+compress' => ( default => 1 );
 has storable => (
     is            => 'rw',
     isa           => 'Bool',
-    documentation => 'Use the Storable module (instead of Treex::PML) for storing into .streex files. Defaults to document->compress, or 0.'
+    documentation => 'Use the Storable module (instead of Treex::PML) for storing into .streex files. Defaults to document->storable.'
 );
 
 # HACKS: Treex::PML::Document->save() cannot take filehandle
@@ -31,6 +31,10 @@ override '_get_filename' => sub {
 
 # If a gzipped file is opened first, the header won't be correct
 override '_open_file_handle' => sub {
+    my ( $self, $filename ) = @_;
+    
+    # This line is the only one needed from super() implementation
+    mkpath( dirname($filename) );
     return;
 };
 
@@ -50,30 +54,31 @@ override '_document_extension' => sub {
     }
 };
 
-sub process_document {
+override '_do_process_document' => sub {
 
     my ( $self, $document ) = @_;
 
-    # prepare the correct file name
-    $self->_prepare_file_handle( $document );
-
     my $filename = $self->_last_filename;
     $document->set_filename($filename);
     $document->save($filename);
     return 1;
-}
+};
 
 1;
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
 Treex::Block::Write::Treex
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
