@@ -1,8 +1,5 @@
 package Treex::Core::Bundle;
-{
-  $Treex::Core::Bundle::VERSION = '0.08664';
-}
-
+$Treex::Core::Bundle::VERSION = '1.20150902';
 use namespace::autoclean;
 
 use Moose;
@@ -272,7 +269,7 @@ sub get_attr {
 # ------- other -------------
 
 sub following {
-    Treex::Core::Node::following(@_);
+    return Treex::Core::Node::following(@_);
 }
 
 __PACKAGE__->meta->make_immutable;
@@ -292,7 +289,7 @@ Treex::Core::Bundle - a set of equivalent sentences in the Treex framework
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
