@@ -1,8 +1,5 @@
 package Treex::Core::BundleZone;
-{
-  $Treex::Core::BundleZone::VERSION = '0.08664';
-}
-
+$Treex::Core::BundleZone::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Common;
 use MooseX::NonMoose;
@@ -93,7 +90,9 @@ sub create_tree {
 
     # pml-typing
     #$tree_root->set_type_by_name( $self->get_document->metaData('schema'), lc($layer) . '-root.type' );
-    $tree_root->set_type_by_name( $self->get_document->metaData('schema'), $tree_root->get_pml_type_name() );
+#    $tree_root->set_type_by_name( $self->get_document->metaData('schema'), $tree_root->get_pml_type_name() );
+
+    $tree_root->fix_pml_type();
 
     # vyresit usporadavaci atribut!
     # TODO: if $tree_root->does('Treex::Core::Role::OrderedTree')
@@ -135,7 +134,7 @@ sub get_tree {
     my $tree      = $self->{trees}->{$tree_name};
 
     if ( not defined $tree ) {
-        log_fatal( "No $tree_name available in the bundle, bundle id=" . $self->get_attr('id') );
+        log_fatal( "No $tree_name available in bundle ".$self->get_bundle->get_attr('id')." in zone " . $self->get_label() );
     }
     return $tree;
 }
@@ -248,7 +247,7 @@ Treex::Core::BundleZone - contains a sentence and its linguistic representations
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 SYNOPSIS
 
