@@ -1,7 +1,5 @@
 package Treex::Core::Config;
-{
-  $Treex::Core::Config::VERSION = '0.08664';
-}
+$Treex::Core::Config::VERSION = '1.20150902';
 use strict;
 use warnings;
 
@@ -12,11 +10,11 @@ use File::Spec;
 use File::Slurp 9999;    # prior versions had different interface
 use Cwd qw(realpath);
 use Treex::Core::Log;
+use Treex::Core::Loader 'load_module';
 use YAML 0.72 qw(LoadFile DumpFile);
 
 # this should be somehow systematized, since there will be probably many switches like this one
 our $debug_run_jobs_locally;    ## no critic (ProhibitPackageVars)
-our %service;                   ## no critic (ProhibitPackageVars)
 
 # 0: Treex::Core::Common::pos_validated_list() called if params needed, skipped otherwise
 # 1: Treex::Core::Common::pos_validated_list() called always
@@ -30,8 +28,9 @@ my $running_in_tred;                        ## no critic (ProhibitUnusedVariable
 sub _load_config {
     my $self     = shift;
     my %args     = @_;
-    my $from     = $args{from} // $self->config_file();
-    my $yaml     = read_file( $from, { err_mode => 'silent' } );
+    my $from     = $args{from} // $self->config_file(); #/
+    return {} if !-e $from;
+    my $yaml     = read_file( $from, { err_mode => 'quiet' } );
     my $toReturn = YAML::Load($yaml);
     return $toReturn // {};                 #rather than undef return empty hashref
 }
@@ -39,7 +38,7 @@ sub _load_config {
 sub _save_config {
     my $self = shift;
     my %args = @_;
-    my $to   = $args{to} // $self->config_file();
+    my $to   = $args{to} // $self->config_file(); #/
     return if ( -e $to && !$dirty );        #skip when config file already exists and no changes made from this run of treex so we won't overwrite existing configuration
     return if ( !scalar %{$config} );       #skip when config is empty
     eval {
@@ -55,7 +54,7 @@ END {
 
 sub config_dir {
     my $self = shift;
-    my $dirname = $ENV{TREEX_CONFIG} // File::Spec->catdir( File::HomeDir->my_home(), '.treex' );    # if evironment variable not set defaults to ~/.treex
+    my $dirname = $ENV{TREEX_CONFIG} // File::Spec->catdir( File::HomeDir->my_home(), '.treex' );  #/ # if evironment variable not set defaults to ~/.treex
     if ( !-e $dirname ) {
         mkdir $dirname;
     }
@@ -186,6 +185,21 @@ sub tmp_dir {
     return $config->{tmp_dir};
 }
 
+sub treex_server_url {
+    return exists $ENV{TREEX_SERVER_URL} ?
+      $ENV{TREEX_SERVER_URL} : $config->{treex_server_url} || 'tcp://127.0.0.1:7338';
+}
+
+sub use_services {
+    return $ENV{USE_SERVICES} if defined $ENV{USE_SERVICES};
+
+    if ($config->{use_services}) {
+        $config->{use_services} = 0 unless load_module('Treex::Service::Role');
+    }
+
+    return $config->{use_services} || 0;
+}
+
 sub _default_tmp_dir {
     my $self      = shift;
     my $dot_treex = File::HomeDir->my_dist_data( 'Treex-Core', { create => 1 } );
@@ -219,7 +233,7 @@ Treex::Core::Config - centralized info about Treex configuration
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 SYNOPSIS
 
