@@ -1,8 +1,5 @@
 package Treex::Core::Document;
-{
-  $Treex::Core::Document::VERSION = '0.08664';
-}
-
+$Treex::Core::Document::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Common;
 use Treex::Core::Config;
@@ -19,6 +16,8 @@ use Scalar::Util qw( weaken reftype );
 
 use PerlIO::via::gzip;
 use Storable;
+use Digest::MD5 qw(md5_hex);
+use Lingua::Interset::FeatureStructure;
 
 has loaded_from => ( is => 'rw', isa => 'Str', default => '' );
 has path        => ( is => 'rw', isa => 'Str' );
@@ -32,6 +31,26 @@ has storable => (
     documentation => 'using Storable with gz compression instead of Treex::PML'
 );
 
+has _hash => ( is => 'rw', isa => 'Str' );
+
+sub get_hash {
+    my $self = shift;
+    if ( ! defined($self->_hash) ) {
+        $Storable::canonical = 1;
+        $self->_set_hash(md5_hex(Storable::nfreeze($self)));
+        $Storable::canonical = 0;
+    }
+    return $self->_hash;
+}
+
+sub set_hash {
+    my ($self, $hash) = @_;
+
+    $self->_set_hash($hash);
+
+    return;
+}
+
 has _pmldoc => (
     isa      => 'Treex::PML::Document',
     is       => 'rw',
@@ -105,7 +124,12 @@ sub build_file_number {
 sub full_filename {
     log_fatal 'Incorrect number of arguments' if @_ != 1;
     my $self = shift;
-    return ( $self->path ? $self->path . "/" : '' ) . $self->file_stem . $self->file_number;
+    my $path = '';
+    if (defined $self->path && $self->path ne ''){
+        $path = $self->path;
+        $path .= '/' if $path !~ m{/$};
+    }
+    return  $path . $self->file_stem . $self->file_number;
 }
 
 sub BUILD {
@@ -133,9 +157,17 @@ sub BUILD {
                 # Treex::PML fails with die.
                 # TODO: we should rather catch the die message and report it via log_fatal
                 $pmldoc = eval {
-                    $factory->createDocumentFromFile( $params_rf->{filename} );
+                    # In r10421, ZŽ added here recover => 1:
+                    # $factory->createDocumentFromFile( $params_rf->{filename}, { recover => 1 });
+                    # However, if the file contains invalid PML (e.g. unknown afun value), the recover=>1 option
+                    # results in returning a $pmldoc which seems to be OK, but it contains no bundles,
+                    # so Treex crashes on subsequent blocks which is misleading for users.
+                    # If we really want to be fault-tolerant, it seems we would need to set Treex::PML::Instance::Reader::STRICT=0,
+                    # but I don't no enough about PML internals and I think it's better to make such errors fatal.
+                    # Martin Popel
+                    $factory->createDocumentFromFile( $params_rf->{filename});
                 };
-                log_fatal "Error while loading " . $params_rf->{filename} . ($@ ? "\n$@" : '')
+                log_fatal "Error while loading " . $params_rf->{filename} . ( $@ ? "\n$@" : '' )
                     if !defined $pmldoc;
             }
         }
@@ -146,14 +178,11 @@ sub BUILD {
         $self->_set_pmldoc($pmldoc);
 
         # ensuring Treex::Core types (partially copied from the factory)
-        my $meta = $self->metaData('pml_root')->{meta};
-        if ( defined $meta->{zones} ) {
-            foreach my $doczone ( map { $_->value() } $meta->{zones}->elements ) {
-
-                # $doczone hashref will be reused as the blessed instance variable
-                Treex::Core::DocZone->new($doczone);
-            }
+        # $doczone hashref will be reused as the blessed instance variable
+        for my $doczone ($self->get_all_zones()){
+            Treex::Core::DocZone->new($doczone);
         }
+
         $self->_rebless_and_index();
     }
 
@@ -161,8 +190,21 @@ sub BUILD {
     foreach my $bundle ( $self->get_bundles ) {
         $bundle->deserialize_wild;
         foreach my $bundlezone ( $bundle->get_all_zones ) {
-            foreach my $node ( map { $_->get_descendants( { add_self => 1 } ) } $bundlezone->get_all_trees ) {
-                $node->deserialize_wild;
+            foreach my $tree ( $bundlezone->get_all_trees ){
+                my $ordered = $tree->type->get_structure_name =~ /[at]-(root|node)/ ? 1 : 0;
+                my $correct_ord = 0;
+                my @nodes = $tree->get_descendants( { add_self => 1, ($ordered ? (ordered => 1) : ()) } );
+                foreach my $node (@nodes){
+                    # normalize ord, so there are no gaps
+                    if ($ordered){
+                        $node->_set_ord($correct_ord);
+                        $correct_ord++;
+                    }
+                    $node->deserialize_wild;
+                    if ( $node->DOES('Treex::Core::Node::Interset') ) {
+                        $node->deserialize_iset;
+                    }
+                }
             }
         }
     }
@@ -195,6 +237,9 @@ sub _rebless_and_index {
                     foreach my $node ( $tree, $tree->descendants ) {    # must still call Treex::PML::Node's API
                         bless $node, "Treex::Core::Node::$layer";
                         $self->index_node_by_id( $node->get_id, $node );
+                        if ($layer eq 'A' && $node->{iset}){
+                            $node->{iset} = Lingua::Interset::FeatureStructure->new(%{$node->{iset}});
+                        }
                     }
                     $tree->_set_zone($zone);
                 }
@@ -369,7 +414,7 @@ sub get_all_node_ids {
     return ( keys %{ $self->_index } );
 }
 
-# ----------------- ACCESS TO BUNDLES ----------------------
+# -------------------------------------- ACCESS TO BUNDLES -------------------
 
 sub get_bundles {
     log_fatal('Incorrect number of arguments') if @_ != 1;
@@ -403,7 +448,7 @@ sub create_bundle {
     return $new_bundle;
 }
 
-# -------------- ACCESS TO ZONES ---------------------------------------
+# ----------------------- ACCESS TO ZONES ------------------------------------
 
 sub create_zone {
     my $self = shift;
@@ -433,6 +478,15 @@ sub create_zone {
     return $new_zone;
 }
 
+sub get_all_zones {
+    my $self = shift;
+    my $meta = $self->metaData('pml_root')->{meta};
+    return if !$meta->{zones};
+    
+    # Each element is a pair [$name, $value]. We need just the values.
+    return map {$_->[1]}  $meta->{zones}->elements;
+}
+
 sub get_zone {
     my $self = shift;
     my ( $language, $selector ) = pos_validated_list(
@@ -441,14 +495,8 @@ sub get_zone {
         { isa => 'Treex::Type::Selector', default => '' },
     );
 
-    my $meta = $self->metaData('pml_root')->{meta};
-    if ( defined $meta->{zones} ) {
-        foreach my $element ( $meta->{zones}->elements ) {
-            my ( undef, $value ) = @$element;    # $name is not needed
-            if ( $value->{language} eq $language and ( $value->{selector} || '' ) eq ( $selector || '' ) ) {
-                return $value;
-            }
-        }
+    foreach my $zone ($self->get_all_zones()) {
+        return $zone if $zone->language eq $language && $zone->selector eq $selector;
     }
     return;
 }
@@ -468,7 +516,7 @@ sub get_or_create_zone {
     return $fs_zone;
 }
 
-# -------------- LOADING AND SAVING ---------------------------------------
+# -------------------- LOADING AND SAVING ------------------------------------
 
 sub load {
     my $self = shift;
@@ -483,8 +531,9 @@ sub save {
 
     if ( $filename =~ /\.streex$/ ) {
         open( my $F, ">:via(gzip)", $filename ) or log_fatal $!;
-        print $F Storable::nfreeze( $self );
+        print $F Storable::nfreeze($self);
         close $F;
+
         # using  Storable::nstore_fd($self,*$F) emits 'Inappropriate ioctl for device'
     }
 
@@ -504,6 +553,9 @@ sub _serialize_all_wild {
         foreach my $bundlezone ( $bundle->get_all_zones ) {
             foreach my $node ( map { $_->get_descendants( { add_self => 1 } ) } $bundlezone->get_all_trees ) {
                 $node->serialize_wild;
+                if ( $node->DOES('Treex::Core::Node::Interset') ) {
+                    $node->serialize_iset;
+                }
             }
         }
     }
@@ -514,34 +566,55 @@ sub retrieve_storable {
     my ( $class, $file ) = @_;    # $file stands for a file name, but it can be also file handle (needed by the TrEd backend for .streex)
 
     my $FILEHANDLE;
-
-    my $stringified_doc;
+    my $opened = 0;
 
     if ( ref($file) and reftype($file) eq 'GLOB' ) {
         $FILEHANDLE = $file;
     }
     else {
-        log_fatal "filename=$file, but Treex::Core::Document->retrieve(\$filename) can be used only for .streex files"
+        log_fatal "filename=$file, but Treex::Core::Document->retrieve_storable(\$filename) can be used only for .streex files"
             unless $file =~ /\.streex$/;
         open $FILEHANDLE, "<:via(gzip)", $file or log_fatal($!);
+        $opened = 1;
     }
 
     my $serialized;
+
     # reading it this way is silly, but both slurping the file or
     #  using Storable::retrieve_fd lead to errors when used with via(gzip)
     while (<$FILEHANDLE>) {
         $serialized .= $_;
     }
+
+    if ( $opened ) {
+        close($FILEHANDLE);
+    }
+
     # my $retrieved_doc = Storable::retrieve_fd(*$FILEHANDLE) or log_fatal($!);
-    my $retrieved_doc = Storable::thaw( $serialized ) or log_fatal $!;
+    my $retrieved_doc = Storable::thaw($serialized) or log_fatal $!;
 
     if ( not ref($file) ) {
         $retrieved_doc->set_loaded_from($file);
-        my ( $volume, $dirs, $file ) = File::Spec->splitpath($file);
-        $retrieved_doc->set_path($volume . $dirs);
+        my ( $volume, $dirs, $file_name ) = File::Spec->splitpath($file);
+        $retrieved_doc->set_path( $volume . $dirs );
+
         # $retrieved_doc->changeFilename($file); # why this doesn't affect the name displayed in TrEd?
     }
 
+    # *.streex files saved before r8789 (2012-05-29) have no PML types with nodes, let's fix it
+    # TODO: delete this hack as soon as no such old streex files are needed.
+    foreach my $bundle ( $retrieved_doc->get_bundles() ) {
+        foreach my $bundlezone ( $bundle->get_all_zones() ) {
+            foreach my $node ( map { $_->get_descendants() } $bundlezone->get_all_trees() ) {
+
+                # skip this hack if we are dealing with a new streex file
+                #return $retrieved_doc if $node->type;
+                # This shortcut does not work since old files have only *some* nodes without types
+                $node->fix_pml_type();
+            }
+        }
+    }
+
     return $retrieved_doc;
 }
 
@@ -563,7 +636,7 @@ Treex::Core::Document - representation of a text and its linguistic analyses in
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
@@ -572,6 +645,10 @@ of natural language sentences (typically, but not necessarily,
 originating from the same text). Attributes (attribute-value pairs)
 can be attached to a document as a whole.
 
+Note that the references from the bundles to the containing document are weak,
+so make sure you always keep a reference to the document in scope to prevent
+the contents of the document from being garbage-collected.
+
 =head1 ATTRIBUTES
 
 C<Treex::Core::Document>'s instances have the following attributes:
@@ -707,7 +784,7 @@ Remove all references to the given node id (calls remove_reference() on each ref
 
 =head2 Serializing
 
-=over 4 
+=over 4
 
 =item my $document = load($filename, \%opts)
 
