@@ -1,7 +1,5 @@
 package Treex::Core::DocumentReader::Base;
-{
-  $Treex::Core::DocumentReader::Base::VERSION = '0.08664';
-}
+$Treex::Core::DocumentReader::Base::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Common;
 with 'Treex::Core::DocumentReader';
@@ -101,7 +99,7 @@ sub new_document {
         foreach my $zone ( @{ $self->zones } ) {
             my $filename = $zone->current_filename;
             ( $volume, $dirs, $file ) = File::Spec->splitpath($filename);
-            my ( $name, $extension ) = $file =~ /([^.]+)(\..+)?/;
+            my ( $name ) = $file =~ /([^.]+)(\..+)?/;
             my $zonelabel = $zone->zone_label;
             my $lang      = $zone->language;
             my $sele      = $zone->selector;
@@ -216,3 +214,4 @@ sub fill_bundle_zone {
 }
 
 1;
+
