@@ -1,8 +1,5 @@
 package Treex::Core::Node;
-{
-  $Treex::Core::Node::VERSION = '0.08664';
-}
-
+$Treex::Core::Node::VERSION = '1.20150902';
 use namespace::autoclean;
 
 use Moose;
@@ -43,6 +40,11 @@ with 'Treex::Core::WildAttr';
 Readonly my $_SWITCHES_REGEX => qr/^(ordered|add_self|(preceding|following|first|last)_only)$/x;
 my $CHECK_FOR_CYCLES = 1;
 
+our $LOG_NEW = 0;
+our $LOG_EDITS = 0;
+# tip: you can use Util::Eval doc='$Treex::Core::Node::LOG_EDITS=1;' in your scenario
+# Note that most attributes are not set by set_attr. See TODO below.
+
 has _zone => (
     is       => 'rw',
     writer   => '_set_zone',
@@ -89,11 +91,20 @@ sub _not_equals {
 
 sub _index_my_id {
     my $self = shift;
-    pos_validated_list( \@_, { isa => 'Any', optional => 1 } );    #TODO
     $self->get_document->index_node_by_id( $self->id, $self );
     return;
 }
 
+sub _caller_signature {
+    my $level = 1;
+    my ($package, $filename, $line) = caller;
+    while ($package =~ /^Treex::Core/){
+        ($package, $filename, $line) = caller $level++;
+    }
+    $package =~ s/^Treex::Block:://;
+    return "$package#$line";
+}
+
 # ---- access to attributes ----
 
 # unlike attr (implemented in Treex::PML::Instance::get_data)
@@ -148,9 +159,32 @@ sub set_attr {
         my $ids = ref($attr_value) eq 'Treex::PML::List' ? $attr_value : [$attr_value];
         $document->index_backref( $attr_name, $self->id, $ids );
     }
+    elsif ($attr_name eq 'alignment'){
+        my $document = $self->get_document();
+        if ($self->{alignment}){
+            my @old_ids = map { $_->{'counterpart.rf'} } @{$self->{alignment}};
+            $document->remove_backref( 'alignment', $self->id, \@old_ids );
+        }
+        if ($attr_value && @$attr_value){
+            my @new_ids = map { $_->{'counterpart.rf'} } @$attr_value;
+            $document->index_backref( $attr_name, $self->id, \@new_ids );
+        }
+    }
+
+    # TODO: most attributes are set by Moose setters,
+    # e.g. $anode->set_form("Hi") does not call set_attr.
+    # We would need to redefine all the setter to fill wild->{edited_by}.
+    if ($LOG_EDITS){
+        my $signature = $self->wild->{edited_by};
+        if ($signature) {$signature .= "\n";}
+        else {$signature = '';}
+        my $a_value = $attr_value // 'undef';
+        $signature .= "$attr_name=$a_value ". $self->_caller_signature();
+        $self->wild->{edited_by} = $signature;
+    }
 
     #simple attributes can be accessed directly
-    return $self->{$attr_name} = $attr_value if $attr_name =~ /^[\w\.]+$/;
+    return $self->{$attr_name} = $attr_value if $attr_name =~ /^[\w\.]+$/ || $attr_name eq '#name';
     log_fatal "Attribute '$attr_name' contains strange symbols."
         . " No XPath like constructs (e.g. 'a/aux.rf[3]') are allowed."
         if $attr_name =~ /[^-\w\/.]/;
@@ -222,24 +256,35 @@ sub get_zone {
         $zone = $self->get_root->_get_zone;    ## no critic (ProtectPrivateSubs)
     }
 
-    if (!$zone) {
-#        $self->get_document->save("pokus2.treex.gz");
-    }
-
     log_fatal "a node (" . $self->id . ") can't reveal its zone" if !$zone;
     return $zone;
 
 }
 
 sub remove {
-    log_fatal 'Incorrect number of arguments' if @_ != 1;
-    my $self = shift;
+    my ($self, $arg_ref) = @_;
     if ( $self->is_root ) {
         log_fatal 'Tree root cannot be removed using $root->remove().'
             . ' Use $zone->remove_tree($layer) instead';
     }
     my $root     = $self->get_root();
     my $document = $self->get_document();
+    
+    my @children = $self->get_children();
+    if (@children){
+        my $what_to_do = 'remove';
+        if ($arg_ref && $arg_ref->{children}){
+            $what_to_do = $arg_ref->{children};
+        }
+        if ($what_to_do =~ /^rehang/){
+            foreach my $child (@children){
+                $child->set_parent($self->get_parent);
+            }
+        }
+        if ($what_to_do =~ /warn$/){
+            log_warn $self->get_address . " is being removed by remove({children=>$what_to_do}), but it has (unexpected) children";
+        }
+    }
 
     # Remove the subtree from the document's indexing table
     my @to_remove = ( $self, $self->get_descendants );
@@ -268,17 +313,25 @@ sub remove {
 
 # Return all nodes that have a reference of the given type (e.g. 'alignment', 'a/lex.rf') to this node
 sub get_referencing_nodes {
-
-    my ( $self, $type ) = @_;
+    my ( $self, $type, $lang, $sel ) = @_;
     my $doc  = $self->get_document;
     my $refs = $doc->get_references_to_id( $self->id );
-
     return if ( !$refs || !$refs->{$type} );
+    if ((defined $lang) && (defined $sel)) {
+        my @ref_filtered_by_tree;
+        if ($sel eq q() ) {
+            @ref_filtered_by_tree = grep { /(a|t)\_tree\-$lang\-.+/; }@{ $refs->{$type} };            
+        }
+        else {
+            @ref_filtered_by_tree = grep { /(a|t)\_tree\-$lang\_$sel\-.+/; }@{ $refs->{$type} };
+        }
+        return map { $doc->get_node_by_id($_) } @ref_filtered_by_tree;
+    }
     return map { $doc->get_node_by_id($_) } @{ $refs->{$type} };
 }
 
 # Remove a reference of the given type to the given node. This will not remove a reverse reference from document
-# index, since it is itself called from when removing reverse references; use the API methods for the individual
+# index, since it is itself called when removing reverse references; use the API methods for the individual
 # references if you want to keep reverse references up-to-date.
 sub remove_reference {
     my ( $self, $type, $id ) = @_;
@@ -288,7 +341,6 @@ sub remove_reference {
         my $links = $self->get_attr('alignment');
 
         if ($links) {
-            my $document = $self->get_document;
             $self->set_attr( 'alignment', [ grep { $_->{'counterpart.rf'} ne $id } @{$links} ] );
         }
     }
@@ -306,6 +358,22 @@ sub remove_reference {
     return;
 }
 
+
+sub fix_pml_type {
+    log_fatal 'Incorrect number of arguments' if @_ != 1;
+    my $self = shift;
+    if ( not $self->type() ) {
+        my $type = $self->get_pml_type_name();
+        if ( not $type ) {
+            log_warn "No PML type recognized for node $self";
+            return;
+        }
+        my $fs_file = $self->get_document()->_pmldoc;
+        $self->set_type_by_name( $fs_file->metaData('schema'), $type );
+    }
+    return;
+}
+
 sub get_pml_type_name {
     log_fatal 'Incorrect number of arguments' if @_ != 1;
     my $self = shift;
@@ -363,7 +431,7 @@ sub create_child {
     # against the "tectogrammatical ideology" and we use it as a temporary hack.
     my %structured_attrs;
     foreach my $attr ( keys %{$arg_ref} ) {
-        if ( $attr =~ m{/} || $attr eq 'mlayer_pos' ) {
+        if ( $attr =~ m{/} || $attr eq 'mlayer_pos' || $attr eq '#name') {
             $structured_attrs{$attr} = delete $arg_ref->{$attr};
         }
     }
@@ -379,10 +447,18 @@ sub create_child {
         $new_node->set_attr( $attr, $structured_attrs{$attr} );
     }
 
-    my $type = $new_node->get_pml_type_name();
-    return $new_node if !defined $type;
-    my $fs_file = $self->get_bundle->get_document()->_pmldoc;
-    $self->set_type_by_name( $fs_file->metaData('schema'), $type );
+#    my $type = $new_node->get_pml_type_name();
+#    return $new_node if !defined $type;
+#    my $fs_file = $self->get_bundle->get_document()->_pmldoc;
+#    $self->set_type_by_name( $fs_file->metaData('schema'), $type );
+
+    $new_node->fix_pml_type();
+
+    # Remember which module (Treex block) and line number in its source code are responsible for creating this node.
+    if ($LOG_NEW){
+        $new_node->wild->{created_by} = $self->_caller_signature();
+    }
+
     return $new_node;
 }
 
@@ -596,6 +672,11 @@ sub is_descendant_of {
     }
     return 0;
 }
+sub dominates {
+    my $self = shift;
+    my $another_node = shift;
+    return $another_node->is_descendant_of($self);
+}
 
 #----------- alignment -------------
 
@@ -611,10 +692,36 @@ sub get_aligned_nodes {
     return ( undef, undef );
 }
 
+sub get_aligned_nodes_by_tree {
+    my ($self, $lang, $selector) = @_;
+    my @nodes = ();
+    my @types = ();
+    my $links_rf = $self->get_attr('alignment');
+    if ($links_rf) {
+        my $document = $self->get_document;
+        foreach my $l_rf (@{$links_rf}) {
+            if ($l_rf->{'counterpart.rf'} =~ /^(a|t)_tree-$lang(_$selector)?-.+$/) {
+                my $n = $document->get_node_by_id( $l_rf->{'counterpart.rf'} );
+                my $t = $l_rf->{'type'};
+                push @nodes, $n;
+                push @types, $t;
+            }
+        }
+        return ( \@nodes, \@types ) if     scalar(@nodes) > 0 ;    
+    }
+    return ( undef, undef );
+}
+
 sub get_aligned_nodes_of_type {
-    my ( $self, $type_regex ) = @_;
+    my ( $self, $type_regex, $lang, $selector ) = @_;
     my @nodes;
-    my ( $n_rf, $t_rf ) = $self->get_aligned_nodes();
+    my ( $n_rf, $t_rf );
+    if ((defined $lang) && (defined $selector)) {
+        ( $n_rf, $t_rf ) = $self->get_aligned_nodes_by_tree($lang, $selector);
+    }
+    else {
+        ( $n_rf, $t_rf ) = $self->get_aligned_nodes();    
+    }    
     return if !$n_rf;
     my $iterator = List::MoreUtils::each_arrayref( $n_rf, $t_rf );
     while ( my ( $node, $type ) = $iterator->() ) {
@@ -628,7 +735,7 @@ sub get_aligned_nodes_of_type {
 sub is_aligned_to {
     my ( $self, $node, $type ) = @_;
     log_fatal 'Incorrect number of parameters' if @_ != 3;
-    return any { $_ eq $node } $self->get_aligned_nodes_of_type( $node, $type ) ? 1 : 0;
+    return ((any { $_ eq $node } $self->get_aligned_nodes_of_type( $type )) ? 1 : 0);
 }
 
 sub delete_aligned_node {
@@ -649,7 +756,7 @@ sub delete_aligned_node {
 sub add_aligned_node {
     my ( $self, $node, $type ) = @_;
     my $links_rf = $self->get_attr('alignment');
-    my %new_link = ( 'counterpart.rf' => $node->id, 'type' => $type );
+    my %new_link = ( 'counterpart.rf' => $node->id, 'type' => $type // ''); #/ so we have no undefs
     push( @$links_rf, \%new_link );
     $self->set_attr( 'alignment', $links_rf );
     return;
@@ -657,7 +764,6 @@ sub add_aligned_node {
 
 # remove invalid alignment links (leading to unindexed nodes)
 sub update_aligned_nodes {
-
     my ($self)   = @_;
     my $doc      = $self->get_document();
     my $links_rf = $self->get_attr('alignment');
@@ -857,23 +963,24 @@ sub following {
     }
 }
 
+# This is copied from Treex::PML::Node.
+# Using Treex::PML::Node::following is faster than recursion
+# and it does not cause "deep recursion" warnings.
 sub descendants {
-    my ( $self ) = @_;
-    return ( map { $_->_descendants_and_self() } $self->children );
+  my $self = $_[0];
+  my @kin = ();
+  my $desc = $self->Treex::PML::Node::following($self);
+  while ($desc) {
+    push @kin, $desc;
+    $desc = $desc->Treex::PML::Node::following($self);
+  }
+  return @kin;
 }
 
-sub _descendants_and_self {
-    my ( $self ) = @_;
-    return ( $self, map { $_->_descendants_and_self() } $self->children );
-}
-
-
 # TODO: How to do this in an elegant way?
 # Unless we find a better way, we must disable two perlcritics
-package Treex::Core::Node::Removed;
-{
-  $Treex::Core::Node::Removed::VERSION = '0.08664';
-}    ## no critic (ProhibitMultiplePackages)
+package Treex::Core::Node::Removed;    ## no critic (ProhibitMultiplePackages)
+$Treex::Core::Node::Removed::VERSION = '1.20150902';
 use Treex::Core::Log;
 
 sub AUTOLOAD {                         ## no critic (ProhibitAutoloading)
@@ -949,7 +1056,7 @@ Treex::Core::Node - smallest unit that holds information in Treex
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
@@ -1058,12 +1165,20 @@ Returns the parent node, or C<undef> if there is none (if C<$node> itself is the
 
 Makes C<$node> a child of C<$parent_node>.
 
-=item $node->remove();
+=item $node->remove({children=>remove});
 
-Deletes a node and the subtree rooted by the given node.
+Deletes a node.
 Node identifier is removed from the document indexing table.
 The removed node cannot be further used.
 
+Optional argument C<children> in C<$arg_ref> can specify
+what to do with children (and all descendants,
+i.e. the subtree rooted by the given node) if present:
+C<remove>, C<remove_warn>, C<rehang>, C<rehang_warn>.
+The default is C<remove> -- remove recursively.
+C<rehang> means reattach the children of C<$node> to the parent of C<$node>.
+The C<_warn> variants will in addition produce a warning.
+
 =item my $root_node = $node->get_root();
 
 Returns the root of the node's tree.
@@ -1177,6 +1292,11 @@ Actually, this is shortcut for C<$node-E<gt>get_siblings({following_only=E<gt>1,
 
 =item my $type = $node->get_pml_type_name;
 
+=item $node->fix_pml_type();
+
+If a node has no PML type, then its type is detected (according
+to the node's location) and filled by the PML interface.
+
 =back
 
 
@@ -1184,19 +1304,27 @@ Actually, this is shortcut for C<$node-E<gt>get_siblings({following_only=E<gt>1,
 
 =over
 
-=item add_aligned_node
+=item $node->add_aligned_node($target, $type)
 
-=item my ($nodes_rf, $types_rf) = $node->get_aligned_nodes();
+Aligns $target node to $node. The prior existence of the link is not checked.
+
+=item my ($nodes_rf, $types_rf) = $node->get_aligned_nodes()
 
 Returns an array containing two array references. The first array contains the nodes aligned to this node, the second array contains types of the links.
 
-=item my @nodes = $node->get_aligned_nodes_of_type($regex_constraint_on_type);
+=item my @nodes = $node->get_aligned_nodes_of_type($regex_constraint_on_type)
 
-=item delete_aligned_node
+Returns a list of nodes aligned to the $node by the specified alignment type.
 
-=item is_aligned_to
+=item $node->delete_aligned_node($target, $type)
 
-=item update_aligned_nodes()
+All alignments of the $target to $node are deleted, if their types equal $type.
+
+=item my $is_aligned = $node->is_aligned_to($target, $regex_constraint_on_type)
+
+Returns 1 if the nodes are aligned, 0 otherwise.
+
+=item $node->update_aligned_nodes()
 
 Removes all alignment links leading to nodes which have been deleted.
 
