@@ -1,23 +1,22 @@
 package Treex::Core::Node::A;
-{
-  $Treex::Core::Node::A::VERSION = '0.08664';
-}
-
+$Treex::Core::Node::A::VERSION = '1.20150902';
 use namespace::autoclean;
 use Moose;
 use Treex::Core::Common;
+use Storable;
 extends 'Treex::Core::Node';
 with 'Treex::Core::Node::Ordered';
 with 'Treex::Core::Node::InClause';
 with 'Treex::Core::Node::EffectiveRelations';
-#with 'Treex::Core::Node::Interset';
+with 'Treex::Core::Node::Interset' => { interset_attribute => 'iset' };
 
 # Original w-layer and m-layer attributes
 has [qw(form lemma tag no_space_after)] => ( is => 'rw' );
 
 # Original a-layer attributes
+# (Only afun and is_parenthesis_root originate from PDT, the rest was added in Treex).
 has [
-    qw(afun is_parenthesis_root edge_to_collapse is_auxiliary)
+    qw(deprel afun is_parenthesis_root edge_to_collapse is_auxiliary translit gloss)
 ] => ( is => 'rw' );
 
 sub get_pml_type_name {
@@ -33,8 +32,8 @@ sub is_coap_root {
 }
 
 sub n_node {
-    my ($self) = @_;
-    my ($first_n_node) = $self->get_referencing_nodes('a.rf'); 
+    my ($self)         = @_;
+    my ($first_n_node) = $self->get_referencing_nodes('a.rf');
     return $first_n_node;
 }
 
@@ -56,6 +55,8 @@ sub get_real_afun
     if ( $afun =~ m/^Aux[PC]$/ )
     {
         my @children = $self->children();
+        # Exclude punctuation children (afun-wise, not POS-tag-wise: we do not want to exclude coordination heads).
+        @children = grep {$_->afun() !~ m/^Aux[XGK]$/} (@children);
         my $n        = scalar(@children);
         if ( $n < 1 )
         {
@@ -117,6 +118,8 @@ sub set_real_afun
     if ( $afun =~ m/^Aux[PC]$/ )
     {
         my @children = $self->children();
+        # Exclude punctuation children (afun-wise, not POS-tag-wise: we do not want to exclude coordination heads).
+        @children = grep {$_->afun() !~ m/^Aux[XGK]$/} (@children);
         my $n        = scalar(@children);
         if ( $n < 1 )
         {
@@ -179,13 +182,11 @@ sub set_real_afun
 #------------------------------------------------------------------------------
 sub copy_atree
 {
-    my $self      = shift;
-    my $target    = shift;
+    my $self   = shift;
+    my $target = shift;
 
-    # TODO probably we should do deepcopy
     # Why is this here ? the attributes of the root node are NOT copied, are they ??
-    my %copy_of_wild = %{$self->wild};
-    $target->set_wild(\%copy_of_wild);
+    $target->set_wild( Storable::dclone( $self->wild ) );
 
     my @children0 = $self->get_children( { ordered => 1 } );
     foreach my $child0 (@children0)
@@ -204,9 +205,8 @@ sub copy_atree
     return;
 }
 
-
 sub copy_attributes {
-    my ($self, $other) = @_;
+    my ( $self, $other ) = @_;
 
     # We should copy all attributes that the node has but it is not easy to figure out which these are.
     # TODO: As a workaround, we list the attributes here directly.
@@ -219,9 +219,12 @@ sub copy_attributes {
         $other->set_attr( $attribute, $value );
     }
 
-    # TODO probably we should do deepcopy
-    my %copy_of_wild = %{$self->wild};
-    $other->set_wild(\%copy_of_wild);
+    # copy values of interset features
+    my $f = $self->get_iset_structure();
+    $other->set_iset($f);
+
+    # deep copy of wild attributes
+    $other->set_wild( Storable::dclone( $self->wild ) );
 
     return;
 }
@@ -230,13 +233,13 @@ sub copy_attributes {
 
 sub get_terminal_pnode {
     my ($self) = @_;
-    my $p_rf = $self->get_attr('p_terminal.rf') or return; 
-    my $doc  = $self->get_document();
+    my $p_rf = $self->get_attr('p_terminal.rf') or return;
+    my $doc = $self->get_document();
     return $doc->get_node_by_id($p_rf);
 }
 
 sub set_terminal_pnode {
-    my ($self, $pnode) = @_;
+    my ( $self, $pnode ) = @_;
     my $new_id = defined $pnode ? $pnode->id : undef;
     $self->set_attr( 'p_terminal.rf', $new_id );
     return;
@@ -286,7 +289,7 @@ sub reset_morphcat {
 # Used only for reading from PCEDT/PDT trees, so far.
 sub get_subtree_string {
     my ($self) = @_;
-    return join '', map { $_->form . ( $_->no_space_after ? '' : ' ' ) } $self->get_descendants( { ordered => 1 } );
+    return join '', map { defined( $_->form ) ? ( $_->form . ( $_->no_space_after ? '' : ' ' ) ) : '' } $self->get_descendants( { ordered => 1 } );
 }
 
 #----------- CoNLL attributes -------------
@@ -301,6 +304,36 @@ sub set_conll_cpos   { return $_[0]->set_attr( 'conll/cpos',   $_[1] ); }
 sub set_conll_pos    { return $_[0]->set_attr( 'conll/pos',    $_[1] ); }
 sub set_conll_feat   { return $_[0]->set_attr( 'conll/feat',   $_[1] ); }
 
+#---------- Morphcat -------------
+
+sub morphcat_pos        { return $_[0]->get_attr('morphcat/pos'); }
+sub morphcat_subpos     { return $_[0]->get_attr('morphcat/subpos'); }
+sub morphcat_number     { return $_[0]->get_attr('morphcat/number'); }
+sub morphcat_gender     { return $_[0]->get_attr('morphcat/gender'); }
+sub morphcat_case       { return $_[0]->get_attr('morphcat/case'); }
+sub morphcat_person     { return $_[0]->get_attr('morphcat/person'); }
+sub morphcat_tense      { return $_[0]->get_attr('morphcat/tense'); }
+sub morphcat_negation   { return $_[0]->get_attr('morphcat/negation'); }
+sub morphcat_voice      { return $_[0]->get_attr('morphcat/voice'); }
+sub morphcat_grade      { return $_[0]->get_attr('morphcat/grade'); }
+sub morphcat_mood       { return $_[0]->get_attr('morphcat/mood'); }
+sub morphcat_possnumber { return $_[0]->get_attr('morphcat/possnumber'); }
+sub morphcat_possgender { return $_[0]->get_attr('morphcat/possgender'); }
+
+sub set_morphcat_pos        { return $_[0]->set_attr( 'morphcat/pos',        $_[1] ); }
+sub set_morphcat_subpos     { return $_[0]->set_attr( 'morphcat/subpos',     $_[1] ); }
+sub set_morphcat_number     { return $_[0]->set_attr( 'morphcat/number',     $_[1] ); }
+sub set_morphcat_gender     { return $_[0]->set_attr( 'morphcat/gender',     $_[1] ); }
+sub set_morphcat_case       { return $_[0]->set_attr( 'morphcat/case',       $_[1] ); }
+sub set_morphcat_person     { return $_[0]->set_attr( 'morphcat/person',     $_[1] ); }
+sub set_morphcat_tense      { return $_[0]->set_attr( 'morphcat/tense',      $_[1] ); }
+sub set_morphcat_negation   { return $_[0]->set_attr( 'morphcat/negation',   $_[1] ); }
+sub set_morphcat_voice      { return $_[0]->set_attr( 'morphcat/voice',      $_[1] ); }
+sub set_morphcat_grade      { return $_[0]->set_attr( 'morphcat/grade',      $_[1] ); }
+sub set_morphcat_mood       { return $_[0]->set_attr( 'morphcat/mood',       $_[1] ); }
+sub set_morphcat_possnumber { return $_[0]->set_attr( 'morphcat/possnumber', $_[1] ); }
+sub set_morphcat_possgender { return $_[0]->set_attr( 'morphcat/possgender', $_[1] ); }
+
 1;
 
 __END__
@@ -370,7 +403,7 @@ Treex::Core::Node::A
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
@@ -476,7 +509,7 @@ This method is specific to the A layer because it contains the list of
 attributes. If we could figure out the list automatically, the method would
 become general enough to reside directly in Node.pm.
 
-=item get_n_node()
+=item n_node()
 
 If this a-node is a part of a named entity,
 this method returns the corresponding n-node (L<Treex::Core::Node::N>).
@@ -484,7 +517,7 @@ If this node is a part of more than one named entities,
 only the most nested one is returned.
 For example: "Bank of China"
 
- $n_node_for_china = $a_node_china->get_n_node();
+ $n_node_for_china = $a_node_china->n_node();
  print $n_node_for_china->get_attr('normalized_name'); # China
  $n_node_for_bank_of_china = $n_node_for_china->get_parent();
  print $n_node_for_bank_of_china->get_attr('normalized_name'); # Bank of China
