@@ -1,7 +1,5 @@
 package Treex::Core::Node::EffectiveRelations;
-{
-  $Treex::Core::Node::EffectiveRelations::VERSION = '0.08664';
-}
+$Treex::Core::Node::EffectiveRelations::VERSION = '1.20150902';
 use Moose::Role;
 
 # with Moose >= 2.00, this must be present also in roles
@@ -44,22 +42,27 @@ sub get_echildren {
         $arg_ref = {};
     }
     log_fatal('Incorrect number of arguments') if @_ > 2;
-    $self->_can_apply_eff($arg_ref) or return $self->get_children();
 
-    # 1) Get my own effective children (i.e. I am their only eff. parent).
-    # These are in my subtree.
-    my @echildren = $self->_get_my_own_echildren($arg_ref);
+    my @echildren;
+    if ( $self->_can_apply_eff($arg_ref) ) {
+        # 1) Get my own effective children (i.e. I am their only eff. parent).
+        # These are in my subtree.
+        @echildren = $self->_get_my_own_echildren($arg_ref);
 
-    # 2) Add shared effective children
-    # (i.e. I am their eff. parent, but not the only one).
-    # This can happen only if I am member of a coordination
-    # and these eff. children are shared modifiers of the coordination.
-    push @echildren, $self->_get_shared_echildren($arg_ref);
+        # 2) Add shared effective children
+        # (i.e. I am their eff. parent, but not the only one).
+        # This can happen only if I am member of a coordination
+        # and these eff. children are shared modifiers of the coordination.
+        push @echildren, $self->_get_shared_echildren($arg_ref);
+    } else {
+        @echildren = $self->get_children();
+    }
 
     # 3) Process eventual switches (ordered=>1, add_self=>1,...)
     #return @echildren if !$arg_ref; TODO this cannot happen now, see $arg_ref = {} if !defined $arg_ref;
     delete $arg_ref->{dive};
     delete $arg_ref->{or_topological};
+    delete $arg_ref->{ignore_incorrect_tree_structure};
     return $self->_process_switches( $arg_ref, @echildren );
 }
 
@@ -69,10 +72,10 @@ sub get_eparents {
         $arg_ref = {};
     }
     log_fatal('Incorrect number of arguments') if @_ > 2;
-    $self->_can_apply_eff($arg_ref) or return $self->get_parent();
 
     # Get effective parents
-    my @eparents = $self->_get_eparents($arg_ref);
+    my @eparents = $self->_can_apply_eff($arg_ref) ?
+        $self->_get_eparents($arg_ref) : ($self->get_parent());
 
     # Process eventual switches (ordered=>1, add_self=>1,...)
     delete $arg_ref->{dive};
@@ -250,6 +253,14 @@ sub get_coap_members {
     return @members;
 }
 
+use List::MoreUtils "any";
+sub is_echild_of {
+    my ($potential_echild, $eparent, $arg_ref) = @_;
+
+    my @echildren = $eparent->get_echildren( $arg_ref );
+    return any { $_ == $potential_echild } @echildren;
+}
+
 1;
 
 __END__
@@ -262,7 +273,7 @@ Treex::Core::Node::EffectiveRelations
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
@@ -272,7 +283,7 @@ This notion is used both
 on the a-layer (L<Treex::Core::Node::A>) and
 on the t-layer (L<Treex::Core::Node::T>).
 
-TODO: explain it, some examples, reference to PDT manual 
+TODO: explain it, some examples, reference to PDT manual
 
 Eg. in the sentence "Martin and Rudolph came", the tree structure is
 
@@ -288,36 +299,36 @@ and C<get_eparents> return when called on various nodes
 
  "came"->get_children()
  # returns "and"
- 
+
  "came"->get_echildren()
  # returns ("Martin", "Rudolph")
- 
+
  "and"->get_children()
  # returns ("Martin", "Rudolph")
- 
+
  "and"->get_echildren()
  # returns ("Martin", "Rudolph") and issues a warning:
  # get_echildren called on coap root ([id_of_and]). Fallback to topological one.
- 
+
  "and"->get_echildren({or_topological => 1})
  # returns ("Martin", "Rudolph") with no warning
 
 
  # PARENTS AND EFFECTIVE PARENTS
- 
+
  "Rudolph"->get_parent()
  # returns "and"
- 
+
  "Rudolph"->get_eparents()
  # returns "came"
- 
+
  "and"->get_parent()
  # returns "came"
- 
+
  "and"->get_eparents()
  # returns "came" and issues a warning:
  # get_eparents called on coap root ([id_of_and]). Fallback to topological one.
- 
+
  "and"->get_eparents({or_topological => 1})
  # returns "came" with no warning
 
@@ -339,7 +350,7 @@ This warning can be supressed by option C<or_topological>.
 
 =item my @effective_children = $node->get_echildren($arg_ref?)
 
-Returns a list of effective children of the C<$node>. It means that 
+Returns a list of effective children of the C<$node>. It means that
 a) instead of coordination/aposition heads, their members are returned
 b) shared modifiers of a coord/apos (technically hanged on the head of coord/apos)
    count as effective children of the members.
@@ -355,7 +366,7 @@ C<dive> is a reference to a subroutine that decides
 whether the given node should be skipped or not.
 Typically this is used for prepositions and subord. conjunctions on a-layer.
 You can set C<dive> to the string C<AuxCP> which is a shortcut
-for C<sub {my $self=shift;return $self->afun =~ /^Aux[CP]$/;}>. 
+for C<sub {my $self=shift;return $self->afun =~ /^Aux[CP]$/;}>.
 
 =item or_topological
 
@@ -411,7 +422,7 @@ L<Treex::Core::Node::get_children()|Treex::Core::Node/get_children>.
 
 If the node is a coordination/apposition head (see
 L<Node::A::is_coap_root()|Node::A/is_coap_root> and
-L<Node::T::is_coap_root()|Node::T/is_coap_root>) a list of all coordinated 
+L<Node::T::is_coap_root()|Node::T/is_coap_root>) a list of all coordinated
 members is returned. Otherwise, the node itself is returned.
 
 OPTIONS
