@@ -1,7 +1,5 @@
 package Treex::Core::Node::Ordered;
-{
-  $Treex::Core::Node::Ordered::VERSION = '0.08664';
-}
+$Treex::Core::Node::Ordered::VERSION = '1.20150902';
 use Moose::Role;
 
 # with Moose >= 2.00, this must be present also in roles
@@ -22,6 +20,11 @@ sub precedes {
     my ( $self, $another_node ) = @_;
     return $self->ord() < $another_node->ord();
 }
+sub follows {
+    log_fatal 'Incorrect number of arguments' if @_ != 2;
+    my ( $self, $another_node ) = @_;
+    return $self->ord() > $another_node->ord();
+}
 
 # Methods get_next_node and get_prev_node are implemented
 # so they can handle deprecated fractional ords.
@@ -62,6 +65,25 @@ sub get_prev_node {
     return $prev_node;
 }
 
+sub get_nodes_between {
+    log_fatal 'Incorrect number of arguments' if @_ != 2;
+    my ($self, $other) = @_;
+
+    # want $self preceding $other
+    if ( $self->ord > $other->ord ) {
+        my $temp = $other;
+        $other = $self;
+        $self = $temp;
+    }
+
+    my @all_nodes = $self->get_root->get_descendants({ordered => 1});
+    my @nodes_between = grep {
+        $_->ord > $self->ord && $_->ord < $other->ord
+    } @all_nodes;
+
+    return @nodes_between;
+}
+
 sub _normalize_node_ordering {
     log_fatal 'Incorrect number of arguments' if @_ != 1;
     my $self = shift;
@@ -219,9 +241,9 @@ sub _shift_to_node {
 }
 
 ###!!! DZ: reverting to 7183, 2011-11-03.
-# This function by Martin is faster than the my own below but it does not always return correct results.
+# This function by Martin is faster than my own below but it does not always return correct results.
 # The key problem could be that it relies on ord values forming a contiguous sequence which might not always be true.
-# Go to devel/normalize_treebanks/nonprojectivity and test that make yields the same numbers for the old and the new method.
+# Go to devel/hamledt/nonprojectivity and test that make yields the same numbers for the old and the new method.
 sub is_nonprojective_does_not_work {
     my ($self) = @_;
 
@@ -301,7 +323,7 @@ Treex::Core::Node::Ordered
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
@@ -343,6 +365,14 @@ or C<undef> if C<$node> is the last one in the tree.
 Return the closest preceding node (according to the ordering attribute)
 or C<undef> if C<$node> is the first one in the tree.
 
+=item my @nodes_in_between = $node->get_nodes_between($other_node);
+
+Return nodes that lie between C<$node> and C<$other_node>, ordered, exclusive.
+(The returned array may be empty.)
+C<$node> and C<$other_node> can be given in any order,
+i.e. the result of C<$node-&gt;get_nodes_between($other_node)>
+is the same as the result of C<$other_node-&gt;get_nodes_between($node)>.
+
 =back
 
 =head2 Reordering nodes
