@@ -1,8 +1,5 @@
 package Treex::Core::Node::P;
-{
-  $Treex::Core::Node::P::VERSION = '0.08664';
-}
-
+$Treex::Core::Node::P::VERSION = '1.20150902';
 use namespace::autoclean;
 
 use Moose;
@@ -23,10 +20,10 @@ has [qw( phrase functions )] => ( is => 'rw' );
 sub get_pml_type_name {
     my ($self) = @_;
 
-    if ( $self->is_root() or $self->phrase ) {
+    if ( $self->is_root() or $self->phrase or ($self->{'#name'} || '') eq 'nonterminal') {
         return 'p-nonterminal.type';
     }
-    elsif ( $self->tag ) {
+    elsif ( $self->tag or ($self->{'#name'} || '') eq 'terminal') {
         return 'p-terminal.type';
     }
     else {
@@ -39,21 +36,49 @@ sub is_terminal {
   return $self->get_pml_type_name eq 'p-terminal.type' ? 1 : 0;
 }
 
+sub create_child {
+    my $self = shift;
+    log_warn 'With Treex::Core::Node::P you should you either create_terminal_child() or create_nonterminal_child() instead of create_child()';
+    return $self->SUPER::create_child(@_);
+}
+
 sub create_nonterminal_child {
     my $self    = shift @_;
+    log_warn 'Adding a child to a terminal p-node ' . $self->id if $self->is_terminal();
+    my $arg_ref;
+    if ( scalar @_ == 1 && ref $_[0] eq 'HASH' ) {
+        $arg_ref = $_[0];
+    }
+    elsif ( @_ % 2 ) {
+        log_fatal "Odd number of elements for create_nonterminal_child";
+    }
+    else {
+        $arg_ref = {@_};
+    }
+    $arg_ref->{'#name'} = 'nonterminal';
     my $fs_file = $self->get_bundle->get_document()->_pmldoc;
-    my $child   = $self->create_child(@_);
+    my $child   = $self->SUPER::create_child($arg_ref);
     $child->set_type_by_name( $fs_file->metaData('schema'), 'p-nonterminal.type' );
-    $child->{'#name'} = 'nonterminal';
     return $child;
 }
 
 sub create_terminal_child {
     my $self    = shift @_;
+    log_warn 'Adding a child to a terminal p-node ' . $self->id if $self->is_terminal();
+    my $arg_ref;
+    if ( scalar @_ == 1 && ref $_[0] eq 'HASH' ) {
+        $arg_ref = $_[0];
+    }
+    elsif ( @_ % 2 ) {
+        log_fatal "Odd number of elements for create_terminal_child";
+    }
+    else {
+        $arg_ref = {@_};
+    }
+    $arg_ref->{'#name'} = 'terminal';
     my $fs_file = $self->get_bundle->get_document()->_pmldoc;
-    my $child   = $self->create_child(@_);
+    my $child   = $self->SUPER::create_child($arg_ref);
     $child->set_type_by_name( $fs_file->metaData('schema'), 'p-terminal.type' );
-    $child->{'#name'} = 'terminal';
     return $child;
 }
 
@@ -72,14 +97,29 @@ sub create_from_mrg {
     $mrg_string =~ s/-LCB-/\{/g;
     $mrg_string =~ s/-RCB-/\}/g;
 
-    # remove extra outer parenthesis
-    $mrg_string =~ s/^\( (\(.+) \)$/$1/;
-
-    # remove one extra non-terminal (ROOT comes from Stanford, S1 comes from Charniak parser)
-    $mrg_string =~ s/^\( (ROOT|S1) (.+) \)$/$2/g;
+    # mrg string should always start with "( ROOT (" as in the Stanford parser.
+    # The ROOT non-terminal has usually just one child "S",
+    # but sometimes (in parsing or in Brown treebank) it has more children,
+    # so the artificial root non-terminal "ROOT" makes sure it is formally a tree (not a forest).
+    # Charniak parser output starts with "( S1 (".
+    # PennTB trees start with "( (".
+    $mrg_string =~ s/^\( (S1 )?\(/( ROOT (/;
+    
     my @tokens = split / /, $mrg_string;
 
     $self->_parse_mrg_nonterminal( \@tokens );
+    
+    # If there is just one child, we can remove the extra ROOT non-terminal.
+    if ($self->get_children == 1){
+        my ($child) = $self->get_children();
+        foreach my $grandchild ($child->get_children) {
+            $grandchild->set_parent($self);
+        }
+        $self->set_phrase($child->phrase);
+        $self->set_functions($child->functions);
+        $child->remove();
+    }
+
     return;
 }
 
@@ -105,7 +145,11 @@ sub _parse_mrg_nonterminal {
 
     # TODO: handle traces correctly
     # Delete trace indices (e.g. NP-SBJ-10 ... -NONE- *T*-10)
+    # Delete suffixes in Brown data, e.g. :"SBJ=1" -> "SBJ", "LOC=2" -> "LOC"
     @label_components = grep { !/^\d+$/ } @label_components;
+    foreach my $comp (@label_components) {
+        $comp =~ s/=\d+$//;
+    }
 
     if (@label_components) {
         $self->set_functions( \@label_components );
@@ -227,7 +271,7 @@ Treex::Core::Node::P
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
