@@ -1,8 +1,5 @@
 package Treex::Core::Node::T;
-{
-  $Treex::Core::Node::T::VERSION = '0.08664';
-}
-
+$Treex::Core::Node::T::VERSION = '1.20150902';
 use namespace::autoclean;
 
 use Moose;
@@ -11,6 +8,7 @@ extends 'Treex::Core::Node';
 with 'Treex::Core::Node::Ordered';
 with 'Treex::Core::Node::InClause';
 with 'Treex::Core::Node::EffectiveRelations';
+with 'Treex::Core::Node::Interset' => { interset_attribute => 'dset' };
 
 # t-layer attributes
 has [
@@ -165,8 +163,21 @@ sub get_anodes {
 
 sub get_coref_nodes {
     my ( $self, $arg_ref ) = @_;
-    my @nodes = ( $self->_get_node_list('coref_gram.rf'), $self->_get_node_list('coref_text.rf') );
-    return $self->_process_switches( $arg_ref, @nodes );
+    
+    # process coreference parameters
+    my $with_types = $arg_ref->{with_types};
+    delete $arg_ref->{with_types};
+    
+    my @gram_nodes = $self->_get_node_list('coref_gram.rf');
+    
+    # textual coreference in PDT2.0 and 2.5 style
+    my @text_nodes = $self->_get_node_list('coref_text.rf');
+    return $self->_process_switches( $arg_ref, (@gram_nodes, @text_nodes) ) if (@text_nodes);
+
+    # textual coreference in PDT3.0 style
+    my $pdt30_text_coref_rf = $self->get_attr('coref_text') // [];
+    my @pdt30_gram_coref = map {{'target_node.rf' => $_->id, 'type' => undef}} @gram_nodes;
+    return $self->_get_pdt30_coref([@pdt30_gram_coref, @$pdt30_text_coref_rf], $with_types, $arg_ref);
 }
 
 sub get_coref_gram_nodes {
@@ -176,7 +187,42 @@ sub get_coref_gram_nodes {
 
 sub get_coref_text_nodes {
     my ( $self, $arg_ref ) = @_;
-    return $self->_get_node_list( 'coref_text.rf', $arg_ref );
+    
+    # process coreference parameters
+    my $with_types = $arg_ref->{with_types};
+    delete $arg_ref->{with_types};
+    
+    # textual coreference in PDT2.0 and 2.5 style
+    my @nodes = $self->_get_node_list( 'coref_text.rf', $arg_ref );
+    return @nodes if (@nodes);
+    
+    # textual coreference in PDT3.0 style
+    my $pdt30_coref_rf = $self->get_attr('coref_text') // [];
+
+    return $self->_get_pdt30_coref($pdt30_coref_rf, $with_types, $arg_ref);
+}
+
+sub _get_pdt30_coref {
+    my ($self, $coref_rf, $with_types, $arg_ref) = @_;
+    
+    my $document = $self->get_document;
+    
+    my @nodes = map {$document->get_node_by_id( $_->{'target_node.rf'} )} @$coref_rf;
+    ## get_node_by_id() will fatally fail if target is not defined!
+    #my @targetrfs = grep {defined($_)} map {$_->{'target_node.rf'}} @{$coref_rf};
+    #my @nodes = map {$document->get_node_by_id($_)} @targetrfs;
+    
+    my @filtered_nodes = $self->_process_switches( $arg_ref, @nodes );
+    return @filtered_nodes if (!$with_types);
+    
+    # return both nodes and types (as list references - similar to alignments)
+    my %node_id_to_index = map {$nodes[$_]->id => $_} 0 .. $#nodes;
+    my @types = map { $_->{'type'} } @$coref_rf;
+    my @filtered_types = map {
+        my $idx = $node_id_to_index{$_->id};
+        defined $idx ? $types[$idx] : undef
+    } @filtered_nodes;
+    return (\@filtered_nodes, \@filtered_types);
 }
 
 # it doesn't return a complete chain, just the members which are accessible
@@ -230,6 +276,11 @@ sub update_coref_nodes {
 
 # ----------- complement nodes -------------
 
+sub get_compl_nodes {
+    my ( $self, $arg_ref ) = @_;
+    return $self->_get_node_list( 'compl.rf', $arg_ref );
+}
+
 sub add_compl_nodes {
     my $self = shift;
     return $self->_add_to_node_list( 'compl.rf', @_ );
@@ -318,6 +369,11 @@ sub set_gram_negation      { return $_[0]->set_attr( 'gram/negation',      $_[1]
 sub set_gram_definiteness  { return $_[0]->set_attr( 'gram/definiteness',  $_[1] ); }
 sub set_gram_diathesis     { return $_[0]->set_attr( 'gram/diathesis',     $_[1] ); }
 
+#------------- valency frame reference -----
+
+sub val_frame_rf { return $_[0]->get_attr('val_frame.rf'); }
+sub set_val_frame_rf { return $_[0]->set_attr('val_frame.rf', $_[1]); }
+
 1;
 
 __END__
@@ -330,7 +386,7 @@ Treex::Core::Node::T
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
@@ -347,7 +403,7 @@ t-layer (tectogrammatical) node
 
 Add auxiliary a-nodes (to C<a/aux.rf>)
 
-=item $anodes = $node->get_anodes()
+=item @anodes = $node->get_anodes()
 
 Return a-nodes (both auxiliary and lexical, ie. C<a/aux.rf> and C<a/lex.rf>)
 
@@ -374,6 +430,7 @@ Remove the specified a-nodes from C<a/aux.rf> (if they are contained in it).
 =item $node->get_coref_nodes()
 
 Return textual and grammatical coreference nodes (from C<coref_gram.rf> and C<coref_text.rf>).
+If the document follows the PDT3.0 annotation style, the same steps as for C<$node->get_coref_text_nodes()> are applied.
 
 =item $node->get_coref_gram_nodes()
 
@@ -381,7 +438,10 @@ Return grammatical coreference nodes (from C<coref_gram.rf>).
 
 =item $node->get_coref_text_nodes()
 
-Return textual coreference nodes (from C<coref_text.rf>).
+Return textual coreference nodes (from C<coref_text.rf> if the document follows the PDT2.0 annotation style).
+If the document follows the PDT3.0 annotation style, the list of nodes is extracted from C<coref_text/target_node.rf>.
+If the C<with_types> parameter is set in C<arg_ref>, two list references are returned: the first one refers to a list of nodes
+extracted from C<coref_text/target_node.rf>, the second one to a list of types from C<coref_text/type>.
 
 =item $node->add_coref_gram_nodes(@nodes)
 
