@@ -1,8 +1,5 @@
 package Treex::Core::TredView;
-{
-  $Treex::Core::TredView::VERSION = '0.08664';
-}
-
+$Treex::Core::TredView::VERSION = '1.20150902';
 # planned to be used from contrib.mac of tred's extensions
 
 use Moose;
@@ -12,13 +9,20 @@ use Treex::Core::TredView::Labels;
 use Treex::Core::TredView::Styles;
 use Treex::Core::TredView::Vallex;
 use Treex::Core::Types;
+use Scalar::Util qw(blessed refaddr);
 use List::Util qw(first);
+use Cache::LRU;
 
 use Treex::Core::Config;
 $Treex::Core::Config::running_in_tred = 1;
 
 has 'grp'       => ( is => 'rw' );
 has 'pml_doc'   => ( is => 'rw' );
+has 'doc_cache' => (
+    is  => 'ro',
+    isa => 'Cache::LRU',
+    default => sub { Cache::LRU->new(size => 20); }
+);
 has 'treex_doc' => ( is => 'rw' );
 has 'tree_layout' => (
     is      => 'ro',
@@ -100,10 +104,19 @@ sub _spread_nodes {
     return ( $mid + 1 ), @lower[ 0 .. $mid ], $node, @lower[ ( $mid + 1 ) .. $#lower ];
 }
 
+sub _pmldoc_equals {
+    my ($self, $fsfile) = @_;
+    return ($self->pml_doc && $fsfile && refaddr($self->pml_doc) == refaddr($fsfile));
+}
+
 sub get_nodelist_hook {
     my ( $self, $fsfile, $treeNo, $currentNode ) = @_;
 
-    return if not $self->pml_doc();    # get_nodelist_hook is invoked also before file_opened_hook
+    #return if not $self->pml_doc();    # get_nodelist_hook is invoked also before file_opened_hook
+
+    unless ($self->_pmldoc_equals($fsfile)) {
+        $self->file_opened_hook($fsfile);
+    }
 
     my $bundle = $fsfile->tree($treeNo);
     bless $bundle, 'Treex::Core::Bundle';    # TODO: how to make this automatically?
@@ -190,7 +203,6 @@ sub get_nodelist_hook {
         }
         my %hide;
         foreach my $node ( grep { $_->isa('Treex::Core::Node::A') } @nodes ) {
-            my $parent = $node->get_parent;
             if ( defined $node->clause_number and $root{$node} ne $node ) {
                 $hide{$node} = 1;
             }
@@ -218,24 +230,32 @@ sub recompute_visualization {
     $self->precompute_tree_depths($bundle);
     $self->precompute_tree_shifts($bundle);
     $self->precompute_visualization($bundle);
+    $self->precompute_value_line($bundle);
     return;
 }
 
 sub file_opened_hook {
-    my ($self) = @_;
-    my $pmldoc = $self->grp()->{FSFile};
+    my ($self, $fsfile) = @_;
+    my $pmldoc = $fsfile || $self->grp()->{FSFile};
+
+    return unless $pmldoc && $pmldoc->metaData('schema') &&
+      $pmldoc->metaData('schema')->get_root_name() eq 'treex_document';
+
+    return if $self->_pmldoc_equals($pmldoc);
 
     $self->pml_doc($pmldoc);
 
-    my $treex_doc;
-    if ( defined $pmldoc->[13]->{_treex_core_document} ) {    # if it comes from storable (i.e., already with moose)
-        $treex_doc = $pmldoc->[13]->{_treex_core_document};
+    my $treex_doc = $self->doc_cache->get(refaddr($self->pml_doc));
+    unless ($treex_doc) {
+        if ( defined $pmldoc->[13]->{_treex_core_document} ) { # if it comes from storable (i.e., already with moose)
+            $treex_doc = $pmldoc->[13]->{_treex_core_document};
 
-        # streex files do not have "wild_dump" filled, but we want to show this in ttred
-        $treex_doc->_serialize_all_wild();
-    }
-    else {
-        $treex_doc = Treex::Core::Document->new( { pmldoc => $pmldoc } );
+            # streex files do not have "wild_dump" filled, but we want to show this in ttred
+            $treex_doc->_serialize_all_wild();
+        } else {
+            $treex_doc = Treex::Core::Document->new( { pmldoc => $pmldoc } );
+        }
+        $self->doc_cache->set(refaddr($self->pml_doc), $treex_doc);
     }
 
     $self->treex_doc($treex_doc);
@@ -250,7 +270,7 @@ sub file_opened_hook {
         # If we don't care about slow loading of the whole file,
         # we can precompute all bundles now, so browsing through bundles
         # will be a bit faster.
-        if ( !$self->fast_loading ) {
+        if ( !$self->fast_loading && !$bundle->{_precomputed} ) {
             $self->precompute_tree_depths($bundle);
             $self->precompute_tree_shifts($bundle);
             $self->precompute_visualization($bundle);
@@ -266,8 +286,12 @@ sub file_opened_hook {
 }
 
 sub get_value_line_hook {
-    my ( $self, undef, $treeNo ) = @_;    # the unused argument stands for $fsfile
-    return if not $self->pml_doc();
+    my ( $self, $fsfile, $treeNo ) = @_;    # the unused argument stands for $fsfile
+    #return if not $self->pml_doc();
+
+    unless ($self->_pmldoc_equals($fsfile)) {
+        $self->file_opened_hook($fsfile);
+    }
 
     my $bundle = $self->pml_doc->tree($treeNo);
     if ( !$bundle->{_precomputed_value_line} ) {
@@ -348,6 +372,28 @@ sub precompute_tree_depths {
     return;
 }
 
+sub node_release_hook {
+    my ($self, $node, $target, $mod) = @_;
+    my @roots = map { $_->get_root } $node, $target;
+    my @zones = map { $_->get_zone } $node, $target;
+
+    if ($self->show_alignment
+        and
+        $roots[0] != $roots[1] and $zones[0] != $zones[1]) {
+
+        if ($node->is_aligned_to($target, 'alignment')) {
+            $node->delete_aligned_node($target, 'alignment');
+        } else {
+            $node->add_aligned_node($target, 'alignment');
+        }
+
+        # TODO: Is there more treexy way to do it?
+        TredMacro::Redraw();
+
+        return 'stop';
+    }
+}
+
 # Has to be run whenever the tree layout changes.
 sub precompute_tree_shifts {
     my ( $self, $bundle ) = @_;
@@ -483,6 +529,7 @@ sub precompute_value_line {
     my ( $self, $bundle ) = @_;
 
     my %alignment = ();
+
     foreach my $zone ( $bundle->get_all_zones() ) {
         foreach my $layer (@layers) {
             if ( $zone->has_tree($layer) ) {
@@ -490,6 +537,7 @@ sub precompute_value_line {
                 foreach my $node ( $root, $root->get_descendants ) {
                     if ( exists $node->{'alignment'} ) {
                         foreach my $ref ( @{ $node->get_attr('alignment') } ) {
+                            next if !defined $ref || ! defined $ref->{'counterpart.rf'};
                             push @{ $alignment{ $node->{id} } }, $self->treex_doc->get_node_by_id( $ref->{'counterpart.rf'} );
                             push @{ $alignment{ $ref->{'counterpart.rf'} } }, $node;
                         }
@@ -648,10 +696,8 @@ sub node_style_hook {
     # alignment
     if ( $self->show_alignment and my $links = $node->attr('alignment') ) {
         foreach my $link (@$links) {
-            if ( exists $self->_displayed_nodes->{ $link->{'counterpart.rf'} } ) {
-                push @target_ids,  $link->{'counterpart.rf'};
-                push @arrow_types, 'alignment';
-            }
+            push @target_ids, $link->{'counterpart.rf'};
+            push @arrow_types, $link->{'type'};
         }
     }
 
@@ -677,6 +723,10 @@ sub node_style_hook {
 
 sub root_style_hook {
     my ( $self, $bundle, $styles ) = @_;
+
+    # Sometimes TrEd calls this on e.g. t-nodes (after node_click_hook)
+    return if !$bundle->isa('Treex::Core::Bundle');
+
     return if $bundle->{_precomputed};
     $self->precompute_tree_depths($bundle);
     $self->precompute_tree_shifts($bundle);
@@ -761,7 +811,7 @@ sub toggle_tree_wrapping {
     my $signature = $self->_tree_type_signature($node);
     $self->tree_type_to_wrap->{$signature} = not $self->tree_type_to_wrap->{$signature};
     print "Toggle wrapping of trees of type: $signature\n";
-
+    return;
 }
 
 sub _tree_type_signature {
@@ -775,13 +825,54 @@ sub run_annotation_command {
     my ( $self, $command, $node ) = @_;
     Treex::Core::TredView::AnnotationCommand::run( $command, $node );
     $self->recompute_visualization($node->get_bundle);
+    return;
 }
 
 
+
+my $HIGHLIGHT_STYLE = '#{Line-decoration:shape=oval;coords=-20,-10,20,10;outline=#ff0000;width=3;dash=5,5 }';
+my $HIGHLIGHT_TEXT = '-background => #ffbbaa';
+my @highlighted = ();
+sub node_click_hook {
+    my ( $self, $node, $modifier, $xevent ) = @_;
+    my $ali_id = eval {$node->wild->{ali_root}};
+    if ($ali_id){
+        # remove all highlighting
+        foreach my $n (@highlighted){
+            $n->{_precomputed_node_style} =~ s/\Q$HIGHLIGHT_STYLE\E//;
+        }
+        @highlighted = ();
+        # add new highlighted nodes (a minimal treelet consistent with alignment)
+        my $al_node = $self->treex_doc->get_node_by_id($ali_id);
+        my @queue = ($node, $al_node);
+        while (@queue){
+            my $n = shift @queue;
+            push @highlighted, $n;
+            push @queue, grep {!$_->wild->{ali_root}} $n->get_children();
+        }
+        foreach my $n (@highlighted){
+            $n->{_precomputed_node_style} .= $HIGHLIGHT_STYLE;
+        }
+        # highlight also the relevant words in the text window above trees
+        my $bundle = $node->get_bundle();
+        my %h; $h{$_}=1 for (@highlighted);
+        foreach my $token (@{$bundle->{_precomputed_value_line}}){
+            pop @$token if $token->[-1] eq $HIGHLIGHT_TEXT;
+            push @$token, $HIGHLIGHT_TEXT if $h{$token->[1]};
+        }
+        TredMacro::Redraw();
+    }
+    return;
+}
+
 1;
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =for Pod::Coverage precompute_visualization
 
 =head1 NAME
@@ -790,7 +881,7 @@ Treex::Core::TredView - visualization of Treex files in TrEd
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
