@@ -1,8 +1,6 @@
 package Treex::Core::TredView::AnnotationCommand;
-{
-  $Treex::Core::TredView::AnnotationCommand::VERSION = '0.08664';
-}
-
+$Treex::Core::TredView::AnnotationCommand::VERSION = '1.20150902';
+use strict;
 use Treex::Core::Log;
 
 sub run {
@@ -16,19 +14,24 @@ sub run {
         print "in AnnotationCommand2\n";
         $node->set_conll_deprel($argument);
     }
+    return;
 }
 
 1;
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
 Treex::Core::TredView::AnnotationCommand - simple command-line annotation interface
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
