@@ -1,8 +1,5 @@
 package Treex::Core::TredView::Labels;
-{
-  $Treex::Core::TredView::Labels::VERSION = '0.08664';
-}
-
+$Treex::Core::TredView::Labels::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::TredView::Common;
 use Treex::Core::Log;
@@ -213,7 +210,7 @@ sub _anode_labels {
         $line1 .= $self->_colors->get( 'error', 1 ) . $node->lemma;
     }
 
-    my $edge_label = $node->afun || $node->conll_deprel;
+    my $edge_label = $node->deprel || $node->afun || $node->conll_deprel;
     my $color = $edge_label && $edge_label ne 'NR' ? $self->_colors->get( 'afun', 1 ) : $self->_colors->get( 'error', 1 );
     my $line2 = $color . ( $edge_label || '!!' );
     if ( $node->is_member ) {
@@ -226,6 +223,12 @@ sub _anode_labels {
             $line2 .= '_' . $self->_colors->get( 'member', 1 ) . ( $1 ? $1 : $2 );
         }
     }
+    if ( $node->translit ) {
+        $line2 .= ' ' . $self->_colors->get( 'translit', 1 ) . $node->translit;
+    }
+    if ( $node->gloss ) {
+        $line2 .= ' ' . $self->_colors->get( 'gloss', 1 ) . $node->gloss;
+    }
 
     my $line3_1 = $node->tag ? $node->tag : '';
     my $line3_2 = $node->lemma ? $node->lemma : '';
@@ -311,11 +314,11 @@ sub _tnode_labels {
     }
     if (@a_nodes) {
         @a_nodes = sort { $a->{ord} <=> $b->{ord} } @a_nodes;
-        @a_nodes = map { ( $_->{type} eq 'lex' ? $self->_colors->get( 'lex', 1 ) : $self->_colors->get( 'aux', 1 ) ) . ( $_->{form} // '' ) } @a_nodes;
+        @a_nodes = map { ( $_->{type} eq 'lex' ? $self->_colors->get( 'lex', 1 ) : $self->_colors->get( 'aux', 1 ) ) . ( $_->{form} // '' ) } @a_nodes; #/
         $line3_1 = join " ", @a_nodes;
     }
 
-    my $line3_2 = $self->_colors->get( 'nodetype', 1 ) . $node->{nodetype};
+    my $line3_2 = $self->_colors->get( 'nodetype', 1 ) . ($node->{nodetype} || '');
     $line3_2 .= $self->_colors->get( 'sempos', 1 ) . '.' . $node->attr('gram/sempos') if $node->attr('gram/sempos');
 
     return [
@@ -364,13 +367,17 @@ sub _pnode_labels {
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
 Treex::Core::TredView::Labels - Labels of tree nodes in Tred
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
