@@ -1,11 +1,10 @@
 package Treex::Core::TredView::Styles;
-{
-  $Treex::Core::TredView::Styles::VERSION = '0.08664';
-}
-
+$Treex::Core::TredView::Styles::VERSION = '1.20150902';
 use Moose;
 use Treex::Core::Log;
 use Treex::Core::TredView::Colors;
+use Treex::Core::TredView::LineStyles;
+
 
 has '_treex_doc' => (
     is       => 'ro',
@@ -19,6 +18,12 @@ has '_colors' => (
     default => sub { Treex::Core::TredView::Colors->new() }
 );
 
+has '_line_styles' => (
+    is      => 'ro',
+    isa     => 'Treex::Core::TredView::LineStyles',
+    default => sub { Treex::Core::TredView::LineStyles->new() }
+);
+
 sub _is_coord {
     my ( $self, $node ) = @_;
     return 0 if $node->get_layer ne 't';
@@ -60,11 +65,14 @@ sub node_style {
 
 sub _anode_style {
     my ( $self, $node ) = @_;
+    my $linestyle = $node->wild->{changed} ? '#{Line-width:5}' : '';
+    $linestyle .= '#{Line-width:5}' if $node->wild->{joined};
+    $linestyle .= '#{Line-fill:#ff0000}' if $node->wild->{unchangeable};
     if ( $node->clause_number ) {
         my $clr = $self->_colors->get_clause_color( $node->clause_number );
-        return '#{Oval-fill:' . $clr . '}' . '#{Line-fill:' . $clr . '}';
+        return '#{Oval-fill:' . $clr . '}' . '#{Line-fill:' . $clr . '}' . $linestyle;
     }
-    return '#{Oval-fill:' . $self->_colors->get('anode') . '}';
+    return '#{Oval-fill:' . $self->_colors->get('anode') . '}' . $linestyle;
 }
 
 sub _tnode_style {
@@ -128,11 +136,11 @@ sub _tnode_style {
     } else {
         $line_coords .= ',p,p';
     }
-
+    
     $style .= $coord_circle if $self->_is_coord($node);
     $style .= "#{Line-width:$line_width}#{Line-fill:$line_color}#{Line-coords:$line_coords}";
     $style .= "#{Line-dash:$line_dash}" if $line_dash;
-
+    $style .= '#{Oval-fill:#00ff00}' if $node->wild->{ali_root};
     return $style;
 }
 
@@ -183,67 +191,71 @@ sub draw_arrows {
     my ( $rotate_prv_snt, $rotate_nxt_snt, $rotate_dfr_doc ) = ( 0, 0, 0 );
 
     foreach my $target_id (@$target_ids) {
-        my $arrow_type = shift @$arrow_types;
+        next if !defined $target_ids || $target_id eq ""; # skip blank IDs
+        # some alignment links do not have their type filled, default to generic alignment
+        my $arrow_type = shift @$arrow_types // 'alignment';         
 
-        my $target_node = $self->_treex_doc->get_node_by_id($target_id);
+        my $target_node
+            = eval { $self->_treex_doc->get_node_by_id($target_id) };
+        if ($target_node) {
+            if ( $node->get_bundle eq $target_node->get_bundle ) {    # same sentence
 
-        if ( $node->get_bundle eq $target_node->get_bundle ) {    # same sentence
+                my $T  = "[?\$node->{id} eq '$target_id'?]";
+                my $X  = "(x$T-xn)";
+                my $Y  = "(y$T-yn)";
+                my $D  = "sqrt($X**2+$Y**2)";
+                my $BX = 'n';
+                my $BY = 'n';
+                my $MX = "((x$T+xn)/2 - $Y*(25/$D+0.12))";
+                my $MY = "((y$T+yn)/2 + $X*(25/$D+0.12))";
+                my $EX = "x$T";
+                my $EY = "y$T";
+                my $K1 = "20 / sqrt(($MX-xn)**2 + ($MY-yn)**2)";
+                my $K2 = "20 / sqrt((x$T-$MX)**2 + (y$T-$MY)**2)";
 
-            my $T  = "[?\$node->{id} eq '$target_id'?]";
-            my $X  = "(x$T-xn)";
-            my $Y  = "(y$T-yn)";
-            my $D  = "sqrt($X**2+$Y**2)";
-            my $BX = 'n';
-            my $BY = 'n';
-            my $MX = "((x$T+xn)/2 - $Y*(25/$D+0.12))";
-            my $MY = "((y$T+yn)/2 + $X*(25/$D+0.12))";
-            my $EX = "x$T";
-            my $EY = "y$T";
-            my $K1 = "20 / sqrt(($MX-xn)**2 + ($MY-yn)**2)";
-            my $K2 = "20 / sqrt((x$T-$MX)**2 + (y$T-$MY)**2)";
+                if ( $self->_is_coord($node) ) {
+                    $BX = "xn-(xn-$MX)*$K1";
+                    $BY = "yn-(yn-$MY)*$K1";
+                }
+                if ( $self->_is_coord($target_node) ) {
+                    $EX = "x$T+($MX-x$T)*$K2";
+                    $EY = "y$T+($MY-y$T)*$K2";
+                }
+                if ( $arrow_type eq 'coindex' ) {
+                    $MX = "((x$T+xn)/2 + $Y*(25/$D+0.12))";
+                    $MY = "((y$T+6+yn)/2 - $X*(25/$D+0.12))";
+                    $EY = "y$T+6";
+                }
 
-            if ( $self->_is_coord($node) ) {
-                $BX = "xn-(xn-$MX)*$K1";
-                $BY = "yn-(yn-$MY)*$K1";
-            }
-            if ( $self->_is_coord($target_node) ) {
-                $EX = "x$T+($MX-x$T)*$K2";
-                $EY = "y$T+($MY-y$T)*$K2";
-            }
-            if ( $arrow_type eq 'coindex' ) {
-                $MX = "((x$T+xn)/2 + $Y*(25/$D+0.12))";
-                $MY = "((y$T+6+yn)/2 - $X*(25/$D+0.12))";
-                $EY = "y$T+6";
+                push @coords, "$BX,$BY,$MX,$MY,$EX,$EY";
             }
+            else {    # should be always the same document, if it exists at all
 
-            push @coords, "$BX,$BY,$MX,$MY,$EX,$EY";
+                my $orientation = $target_node->get_bundle->get_position - $node->get_bundle->get_position;
+                $orientation = $orientation > 0 ? 'right' : 'left';
+                if ( $orientation =~ /left|right/ ) {
+                    if ( $orientation eq 'left' ) {
+                        log_info "ref-arrows: Preceding sentence\n" if $main::macroDebug;
+                        push @coords, "n,n,n-30,n+$rotate_prv_snt";
+                        $rotate_prv_snt += 10;
+                    }
+                    else {    #right
+                        log_info "ref-arrows: Following sentence\n" if $main::macroDebug;
+                        push @coords, "n,n,n+30,n+$rotate_nxt_snt";
+                        $rotate_nxt_snt += 10;
+                    }
+                }
+            }
         }
-        else {    # should be always the same document, if it exists at all
-
-            my $orientation = $target_node->get_bundle->get_position - $node->get_bundle->get_position - 1;
-            $orientation = $orientation > 0 ? 'right' : ( $orientation < 0 ? 'left' : 0 );
-            if ( $orientation =~ /left|right/ ) {
-                if ( $orientation eq 'left' ) {
-                    log_info "ref-arrows: Preceding sentence\n" if $main::macroDebug;
-                    push @coords, "n,n,n-30,n+$rotate_prv_snt";
-                    $rotate_prv_snt += 10;
-                }
-                else {    #right
-                    log_info "ref-arrows: Following sentence\n" if $main::macroDebug;
-                    push @coords, "n,n,n+30,n+$rotate_nxt_snt";
-                    $rotate_nxt_snt += 10;
-                }
-            }
-            else {
-                log_info "ref-arrows: Not found!\n" if $main::macroDebug;
-                push @coords, "n,n,n+$rotate_dfr_doc,n-25";
-                $rotate_dfr_doc += 10;
-            }
+        else {
+            log_info "ref-arrows: Not found!\n" if $main::macroDebug;
+            push @coords, "n,n,n+$rotate_dfr_doc,n-25";
+            $rotate_dfr_doc += 10;
         }
 
         push @tags, $arrow_type;
         push @colors, ( $self->_colors->get($arrow_type) );
-        push @dash, ( $arrow_type eq 'alignment' ? '5,3' : '' );
+        push @dash, ( $self->_line_styles->dash_style($arrow_type) );
     }
 
     $line->{-coords} ||= 'n,n,p,p';
@@ -268,13 +280,17 @@ sub draw_arrows {
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
 Treex::Core::TredView::Styles - Styling of trees in Tred (how they look)
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
@@ -314,6 +330,6 @@ Josef Toman <toman@ufal.mff.cuni.cz>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright © 2011 by Institute of Formal and Applied Linguistics, Charles University in Prague
+Copyright © 2011-2012 by Institute of Formal and Applied Linguistics, Charles University in Prague
 
 This module is free software; you can redistribute it and/or modify it under the same terms as Perl itself.
