@@ -1,8 +1,5 @@
 package Treex::Core::TredView::TreeLayout;
-{
-  $Treex::Core::TredView::TreeLayout::VERSION = '0.08664';
-}
-
+$Treex::Core::TredView::TreeLayout::VERSION = '1.20150902';
 use Moose;
 use Moose::Util::TypeConstraints;
 
@@ -79,7 +76,7 @@ sub load_layouts {
     $self->{_layouts_loaded} = 1;
 
     my $filename = TredMacro::FindMacroDir('treex') . '/.layouts.cfg';
-    open my $CFG, '<:utf8', $filename or return;
+    open my $CFG, '<:encoding(utf-8)', $filename or return;
 
     while (<$CFG>) {
         chomp;
@@ -108,7 +105,7 @@ sub save_layouts {
     return unless ref(TredMacro::GUI()) eq 'TrEd::Window';
 
     my $filename = TredMacro::FindMacroDir('treex') . '/.layouts.cfg';
-    open my $CFG, '>:utf8', $filename or die $!;
+    open my $CFG, '>:encoding(utf-8)', $filename or die $!;
 
     while ( my ( $label, $cols ) = each %{ $self->_layouts } ) {
         my %coords = ();
@@ -436,13 +433,17 @@ sub conf_dialog {
 
 __END__
 
+=pod
+
+=encoding utf-8
+
 =head1 NAME
 
 Treex::Core::TredView::TreeLayout - Layout of trees in Tred
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =head1 DESCRIPTION
 
