@@ -1,7 +1,5 @@
 package Treex::Core::WildAttr;
-{
-  $Treex::Core::WildAttr::VERSION = '0.08664';
-}
+$Treex::Core::WildAttr::VERSION = '1.20150902';
 use Moose::Role;
 
 use Treex::Core::Log;
@@ -29,7 +27,9 @@ sub wild {
 sub _wild_dump {
     my ($self) = @_;
     if ( $self->isa('Treex::Core::Document') ) {
-        return $self->metaData('pml_root')->{meta}->{wild_dump},
+        my $metadata = $self->metaData('pml_root');
+        my $meta = $metadata->{meta};
+        return $meta->{wild_dump};
     }
     else {
         return $self->{wild_dump};
@@ -82,7 +82,7 @@ Treex::Core::WildAttr - role for arbitrary attributes of Treex objects
 
 =head1 VERSION
 
-version 0.08664
+version 1.20150902
 
 =for test_synopsis my ($node, $value);
 =head1 SYNOPSIS
